/*
	$Id: AnnotationTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.Annotation;
import net.sf.basedb.core.AnnotationSet;
import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Sample;
import net.sf.basedb.core.Type;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>SampleAnnotation</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class AnnotationTransfer extends Transfer
{
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.AnnotationTransfer");
	
	/**
	 
	 */
	private SampleTransfer sampleT = null;
	/**
	 
	 */
	private AnnotationTypeTransfer annotationTypeT = null;

	/**
	Constructs a new AnnotationTransfer.
	
	@param sampleT A finished SampleTransfer
	@param annotationTypeT A finished AnnotationTypeTransfer
	 */
	AnnotationTransfer(SampleTransfer sampleT,
			AnnotationTypeTransfer annotationTypeT)
	{
		log.debug("Checking that necessary transfers have been done before proceeding with Annotations");
		if (sampleT.isDone() && annotationTypeT.isDone())
		{
			this.sampleT = sampleT;
			this.annotationTypeT = annotationTypeT;
		}
		else
		{
			if (!sampleT.isDone()) log.debug("Dependency: Samples haven't been transfered");
			if (!annotationTypeT.isDone()) log.debug("Dependency: AnnotationTypes haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring Annotations...");
		int count = getInt("countSampleAnnotations");
		initIdMap(count);
		startProgress(count, "Annotations");
		runUnBatched(
				prepareStatementFromFile("selectSampleAnnotations"), 0, getBatchSize());
		done();
		log.info("Transferring Annotations...DONE");
	}

	/**
	Creates an {@link Annotation} from the current position in the
	<code>ResultSet</code>. If the AnnotationType is of an unknown type
	the application will stop.
	
	@param rs <code>ResultSet</code> containing the data for this item.
	@throws BaseException If AnnotationType or Sample is missing.
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected void createItem(ResultSet rs) throws BaseException, SQLException
	{
		// This dc could be global if the performance is low, but for now it's
		// sufficient
		DbControl dc = newDbControl();
		Sample sample = sampleT.getSampleById(dc, rs.getInt(1));
		if (sample != null)
		{
			AnnotationSet set = sample.getAnnotationSet();
			AnnotationType annotationType = annotationTypeT.getAnnotationTypeById(
					dc, rs.getInt(2));
			if (set != null && annotationType != null)
			{
				Annotation annotation = set.getAnnotation(annotationType);
				Type type = annotationType.getValueType();				
				try
				{
					String value = rs.getString(3);
					switch (type)
					{
						case INT:
							annotation.setValue(new Integer(value));
							break;
						case FLOAT:
							annotation.setValue(new Float(value));
							break;
						case TEXT:
							annotation.setValue(new String(value));
							break;
						case STRING:
							annotation.setValue(new String(value));
							break;
						default:
							log.error("Unknown Type value: " + type);
							stop();
					}
					dc.saveItem(annotation);
					if (log.isDebugEnabled())
					{
						log.debug("Transfered Annotation: " + value + " of type " + type);
					}
				}
				catch (InvalidDataException e)
				{
					// Annotations with invalid data are not migrated
					log.error("Annotation: " + annotation + " has invalid data and is not migrated");
					if (log.isDebugEnabled()) log.debug("\t" + e);
				}
				catch (NumberFormatException e)
				{
					// Annotations with invalid data are not migrated
					log.error("Annotation: " + annotation + " has a number with invalid format and is not migrated");
					if (log.isDebugEnabled()) log.debug("\t" + e);
				}
			}
		}
		dc.commit();		
	}
}
