/*
	$Id: LabeledExtractTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Extract;
import net.sf.basedb.core.Label;
import net.sf.basedb.core.LabeledExtract;
import net.sf.basedb.core.MeasuredBioMaterial;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>LabeledExtract</dd>
<dt><b>Unused columns</b></dt>
<dd>addedDate, quantityLeft</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li></li>
</ul>
</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class LabeledExtractTransfer extends MeasuredBioMaterialTransfer
{
	private ExtractTransfer extractT = null;
	private LabelTransfer labelT = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.LabeledExtractTransfer");

	/**
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param extractT A finished {@link ExtractTransfer}
	@param labelT A finished {@link LabelTransfer}
	 */
	LabeledExtractTransfer(UserTransfer userT, GroupTransfer groupT,
			ProtocolTransfer protocolT, ExtractTransfer extractT,
			LabelTransfer labelT)
	{
		super(userT, groupT, protocolT);
		log.debug("Checking that necessary transfers have been done before proceeding with LabeledExtract");
		if (extractT.isDone() && labelT.isDone())
		{
			this.extractT = extractT;
			this.labelT = labelT;
		}
		else
		{
			if (!extractT.isDone()) log.debug("Dependency: Extracts haven't been transfered");
			if (!labelT.isDone()) log.debug("Dependency: Labels haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring LabeledExtracts...");
		int count = getInt("countLabeledExtracts");
		initIdMap(count);
		startProgress(count, "LabeledExtracts");
		run(prepareStatementFromFile("selectLabeledExtracts"), 0, 1000);
		done();
		log.info("Transferring LabeledExtracts...DONE");
	}

	/**
	Creates a {@link LabeledExtract} from the current position in
	<code>ResultSet</code>.
	
	@param dc {@link DbControl} used to transfer this item
	@param rs <code>ResultSet</code> containing the data for this item.
	@return <code>null</code> the LabeledExtract is already saved.
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	protected LabeledExtract createItem(DbControl dc, ResultSet rs)
			throws BaseException, SQLException
	{
		Integer labelId = labelT.getBase2Id(rs.getInt(13));
		if (labelId != null)
		{
			LabeledExtract labeledExtract = LabeledExtract.getNew(
					dc, Label.getById(dc, labelId));
			setMeasuredBioMaterialData(
					dc, (MeasuredBioMaterial) labeledExtract, rs);
			Extract extract = extractT.getExtractById(dc, rs.getInt(12));
			if (extract != null)
			{
				if (log.isDebugEnabled()) log.debug("Setting extract: " + extract);
				labeledExtract.setExtract(extract, rs.getFloat(10));
			}
			if (log.isDebugEnabled())
			{
				log.debug("Transfered LabeledExtract: " + labeledExtract.getName() + "\tBASE1 id(" + rs.getInt(1) + ")");
				log.debug("--------------------------------------------------------------------------");
			}
			return labeledExtract;
		}
		else
		{
			log.warn(rs.getString(13) + "  ->  " + labelId + " for "
					+ rs.getString(6));
		}
		return null;
	}
	
	/**
	Returns a Labeled extract from a base1 id.
	
	@param dbc DbControl
	@param base1LabeledExtractId Base1 labeled extract id
	 */
	public LabeledExtract getLabeledExtractById(DbControl dbc, int base1LabeledExtractId)
	{
		try
		{
			Integer id = getBase2Id(base1LabeledExtractId);
			return id == null ? null : LabeledExtract.getById(dbc, id);
		}
		catch (BaseException e)
		{
			log.warn("Couldn't get labeled extract from BASE1 by id: " + base1LabeledExtractId);
			if (log.isDebugEnabled())
			{
				log.debug("\t" + e);
			}
		}
		return null;
	}
	
}
