/*
	$Id: MappedPlateTransfer.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Jari Häkkinen, Nicklas Nordborg, Gregory Vincic
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson, Gregory Vincic
	Copyright (C) 2007 Jari Häkkinen, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.clients.migrate;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.CommonItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.ItemQuery;
import net.sf.basedb.core.Plate;
import net.sf.basedb.core.PlateMapping;
import net.sf.basedb.core.PlateType;
import net.sf.basedb.core.query.Expressions;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Restrictions;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
<dl>
<dt><b>Base 1 tables</b></dt>
<dd>PlateParent, Plate, PlateType</dd>
<dt><b>Unused columns</b></dt>
<dd><b>Plate</b>: addedDate, wizzzard</dd>
<dt><b>Notes</b></dt>
<dd>
<ul>
<li>Mapped plates are saved one at a time.</li>
<li>When all are saved those plates that need destroying are destroyed.</li>
<li>In this transfer it's assumed that a plate in the database cannot have a
smaller id than it's parent. This should only occour in rare cases when the
ParentPlate table has been filled. That is if you have plates with id's
larger than 2147483647.</li>
</ul>
</dd>
</dl>

@author Gregory, Martin
@version 2.0
@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/

public class MappedPlateTransfer extends CommonItemTransfer
{
	/**
	Indicates which plates should be destroyed.
	 */
	private Map<Integer, Boolean> destroyedPlates = null;
	private UnmappedPlateTransfer unmappedPlateT = null;
	private Map<Integer, Parents> plateMappings = null;
	private PlateTypeTransfer plateTypeT = null;
	
	/**
	Logger for this class
	 */
	private static final org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger("net.sf.basedb.clients.migrate.MappedPlateTransfer");

	/**
	Constructs a new MappedPlateTransfer.
	
	@param userT A finished {@link UserTransfer}
	@param groupT A finished {@link GroupTransfer}
	@param plateTypeT A finished {@link PlateTypeTransfer}
	@param reporterT A finished {@link ReporterTransfer}
	@param unmappedPlateT A finished {@link UnmappedPlateTransfer}
	 */
	MappedPlateTransfer(UserTransfer userT, GroupTransfer groupT,
			PlateTypeTransfer plateTypeT, ReporterTransfer reporterT,
			UnmappedPlateTransfer unmappedPlateT)
	{
		super(userT, groupT);
		log.debug("Checking that necessary transfers have been done before proceeding with MappedPlate");
		if (reporterT.isDone() && unmappedPlateT.isDone())
		{
			this.destroyedPlates = unmappedPlateT.getDestroyedPlates();
			this.unmappedPlateT = unmappedPlateT;
			this.plateTypeT = plateTypeT;
		}
		else
		{
			if (!reporterT.isDone()) log.debug("Dependency: Reporters haven't been transfered");
			if (!unmappedPlateT.isDone()) log.debug("Dependency: Unmapped plates haven't been transfered");
			dependencyFailure();
		}
	}

	public void start()
	{
		log.info("Transferring MappedPlates...");
		startProgress(getInt("countDistinctPlateParents"), "MappedPlates");
		log.info("Create plate mappings");
		createPlateMappings();
		runUnBatched(prepareStatementFromFile("selectMappedPlates"), 0, getBatchSize());
		destroyPlates();
		progress.stop();
		done();
		log.info("Transferring MappedPlates...DONE");
	}

	/**
	This method saves parents for each plate in an array for easy access
	later in {@link #createItem(ResultSet)}.
	 */
	private void createPlateMappings()
	{
		try
		{
			PreparedStatement ps = prepareStatementFromFile("selectPlateParents");
			ResultSet rs = ps.executeQuery();
			plateMappings = new HashMap<Integer, Parents>();
			Parents parents = null;
			while (rs.next())
			{
				Integer plateId = new Integer(rs.getInt(1));
				if (!plateMappings.containsKey(plateId))
				{
					parents = new Parents(rs.getInt(3) == 0 ? false : true);
				}
				else
				{
					parents = plateMappings.get(plateId);
				}
				// If plates have more than one parent their positions are 1,2,3
				// or 4 in BASE 2 we use position numbers 0,1,2 and 3
				// If position is 0 then it's a 1-1 mapping
				int position = rs.getInt(3);
				int parent = rs.getInt(2);
				parents.addPlateId(parent, position);
				plateMappings.put(plateId, parents);
				if (log.isDebugEnabled()) 
				{
					log.debug("Mapped Plate with BASE1 id: " + plateId + " to parent with BASE1 id: " + parent + " at position " + position);
				}
			}
		}
		catch (SQLException e)
		{
			log.error("FAILED when executing SQL statement", e);
			stop();
		}
	}

	/**
	@param rs ResultSet containing data for the new plate
	@throws BaseException
	@throws SQLException If current ResultSet position is invalid or number
	        of columns in the select statement is to low.
	 */
	void createItem(ResultSet rs) throws BaseException, SQLException
	{
		ArrayList<Plate> sourcePlates = new ArrayList<Plate>();
		int plateId = rs.getInt(1);
		Parents parents = plateMappings.get(new Integer(plateId));
		int wells = rs.getInt(15);
		
		DbControl b2c = newDbControl();
		PlateMapping plateMapping = null;
		// Create mapped plate
		if (!parents.multipleParents)
		{
			if (log.isDebugEnabled())
			{
				log.debug("Mapped plate has " + wells + " wells");				
			}
			switch (wells)
			{
				// 1 -> 1 mapping
				case 96:
				{
					// Use 1x96 -> 1x96 mapping
					plateMapping = findPlateMapping(b2c, 1, 8, 12, 1, 8, 12);
					log.debug("Using 1x96 -> 1x96 mapping");
					break;
				}
				case 384:
				{
					// Use 1x384 -> 1x384 mapping
					plateMapping = findPlateMapping(b2c, 1, 16, 24, 1, 16, 24);
					log.debug("Using 1x384 -> 1x384 mapping");
					break;
				}
				default:
				{
					log.warn("Error: Unknown number of wells " + rs.getInt(15));
					sourcePlates = null;
				}
			}
		}
		else
		{
			/*
			 * Only allowed 4 -> 1 mapping in base 1 is 4x96 -> 1x384 mapping
			 */
			plateMapping = findPlateMapping(b2c, 4, 8, 12, 1, 16, 24);
			log.debug("Using 4x96 -> 1x384 mapping");
		}
		Integer plateTypeId = plateTypeT.getBase2Id(rs.getInt(9));
		PlateType plateType = PlateType.getById(b2c, plateTypeId);
		Integer[] sourcePlateIds = parents.list();
		for (Integer sourcePlateId : sourcePlateIds)
		{
			Plate sourcePlate = null;
			if (sourcePlateId != null)
			{
				Integer base2Id = getBase2Id(sourcePlateId);
				if (base2Id != null) sourcePlate = Plate.getById(b2c, base2Id);
			}
			sourcePlates.add(sourcePlate);
		}
		if (plateMapping != null)
		{
			ArrayList<Plate> newPlates = (ArrayList<Plate>) plateMapping.apply(
					sourcePlates, plateType);
			for (Plate newPlate : newPlates)
			{
				setCommonItemData(b2c, (CommonItem) newPlate, rs);
				destroyedPlates.put(rs.getInt(1), rs.getBoolean(10));
				newPlate.setBarcode(rs.getString(11));
				// There are only 1 -> 1 or N -> 1 mappings
				b2c.saveItem(newPlate);
				commit(b2c);
				if (log.isDebugEnabled())
				{
					log.debug("Transfered mapped plate: " + newPlate.getName() + "\tBASE1 id(" + plateId + ")\tBASE2 id(" + newPlate.getId() + ")");
					log.debug("--------------------------------------------------------------------------");
				}
				unmappedPlateT.mapId(plateId, newPlate.getId());
			}
		}
	}

	
	public PlateMapping findPlateMapping(DbControl dc, int sourcePlates, int sourceRows, int sourceColumns,
			int destinationPlates, int destinationRows, int destinationColumns)
	{
		ItemQuery<PlateMapping> query = PlateMapping.getQuery();
		query.restrict(Restrictions.eq(Hql.property("sourceCount"), Expressions.integer(sourcePlates)));
		query.restrict(Restrictions.eq(Hql.property("sourceGeometry.rows"), Expressions.integer(sourceRows)));
		query.restrict(Restrictions.eq(Hql.property("sourceGeometry.columns"), Expressions.integer(sourceColumns)));
		query.restrict(Restrictions.eq(Hql.property("destinationCount"), Expressions.integer(destinationPlates)));
		query.restrict(Restrictions.eq(Hql.property("destinationGeometry.rows"), Expressions.integer(destinationRows)));
		query.restrict(Restrictions.eq(Hql.property("destinationGeometry.columns"), Expressions.integer(destinationColumns)));
		
		List<PlateMapping> result = query.list(dc);
		return result.size() == 0 ? null : result.get(0);
	}
	
	/**
	After all plates have been transferred we destroy those that have the
	destroyed flag set.
	 */
	public void destroyPlates()
	{
		log.info("Destroying plates that have the destroyed flag set");
		try
		{
			DbControl b2c = newDbControl();
			for (Map.Entry<Integer, Boolean> entry : destroyedPlates.entrySet())
			{
				if (entry.getValue())
				{
					Plate plate = Plate.getById(b2c, getBase2Id(entry.getKey()));
					plate.setDestroyed(true);
				}
			}
			b2c.commit();
		}
		catch (Exception e)
		{
			log.error("FAILED destroying plates that have the destroyed flag set", e);
			stop();
		}
	}

	/**
	Returns a Plate from the given base 1 plate id.
	
	@param dc <code>DbControl</code> used to get the plate from
	@param base1PlateId
	@throws BaseException if a plate with id base1PlateId doesn't exist.
	 */
	Plate getPlateById(DbControl dc, int base1PlateId) throws BaseException
	{
		Integer plateId = getBase2Id(base1PlateId);
		return plateId == null ? null : Plate.getById(dc, plateId);
	}
	
	public Integer getBase2Id(int base1PlateId)
	{
		Integer id = super.getBase2Id(base1PlateId);
		if (id == null) id = unmappedPlateT.getBase2Id(base1PlateId);
		return id;
	}

	private class Parents
	{
		boolean multipleParents = false;
		Integer[] parents = null;

		Parents(boolean multipleParents)
		{
			this.multipleParents = multipleParents;
			parents = multipleParents ? new Integer[4] : new Integer[1];
		}

		public void addPlateId(int id, int position)
		{
			if (position > 0)
			{
				position--;
			}
			parents[position] = id;
		}

		public Integer[] list()
		{
			return parents;
		}
	}
}
