/*
	$Id: PropertyValue.java 5391 2010-08-18 09:55:17Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.File;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.PermissionDeniedException;
import net.sf.basedb.core.Removable;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.clients.web.Base;
import net.sf.basedb.clients.web.util.HTML;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
	This tag is used to display the property of a {@link BasicItem}.
	<p>
	<b>Example:</b>
	<pre>
   &lt;%@ taglib prefix="base" uri="/WEB-INF/base.tld" %&gt;
   &lt;base:propertyvalue item="<%=user%>" property="name" /&gt;
</pre>

	<p>
	<b>Syntax:</b>
	<pre>
   &lt;base:propertyvalue
      item=...
      property=...
      href=...
      tooltip=...
   &gt;
</pre>
	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>item</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The {@link BasicItem} object to get the property from.
		</td>
	</tr>
	<tr>
		<td>property</td>
		<td>-</td>
		<td>yes</td>
		<td>
			The name of the property. See [@link Metadata#getPropertyValue(BasicItem,String)}
			for a description.
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different buttons.
		</td>
	</tr>
	<tr>
		<td>href</td>
		<td>-</td>
		<td>no</td>
		<td>
			If supplied, the text will be linked to the specified URL.
		</td>
	</tr>
	<tr>
		<td>tooltip</td>
		<td></td>
		<td>no</td>
		<td>
			A short explanation that will show up as a tooltip.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
*/
public class PropertyValue
	extends TagSupport
{

	/**
		
	*/
	private static final long serialVersionUID = -92637960053523649L;

	/**
		The item. Should be BasicItem or BasicData.
	*/
	private transient Object item;
	
	/**
		A DbControl object.
	*/
	private transient DbControl dc;
	
	/**
		The name of the property.
	*/
	private String property;
	
	/**
	 	A link to the property item or not.
	*/
	private boolean enablePropertyLink = true;
	
	/**
		Use for files only, if the view/download link should be enabled or not.
	*/
	private boolean enableDownloadLink = true;
	
	/**
		An optional tooltip.
	*/
	private String tooltip = null;

	/**
	 	Possible to open an edit popup or not with the linked property
	 */
	private boolean enableEditLink = true;
	
	/**
		What to show if the value is null.
	*/
	private String nullText = "<i>- none -</i>";
	
	/*
		Taglib initialization methods
	*/
	public void setDbcontrol(DbControl dc)
	{
		this.dc = dc;
	}
	
	public DbControl getDbcontrol()
	{
		if (dc == null && item instanceof BasicItem)
		{
			return ((BasicItem)item).getDbControl();
		}
		return dc;
	}
	
	public void setItem(Object item)
	{
		// setItem can do metaData = Metadata.getInstance(item.getClass())
		this.item = item;
	}
	public Object getItem()
	{
		return item;
	}
	
	public void setProperty(String property)
	{
		this.property = property;
	}
	public String getProperty()
	{
		return property;
	}
	
	public void setEnablePropertyLink(boolean enabled)
	{
		this.enablePropertyLink = enabled;
	}
	public boolean getEnablePropertyLink()
	{
		return enablePropertyLink;
	}

	public void setEnableDownloadLink(boolean enabled)
	{
		this.enableDownloadLink = enabled;
	}
	public boolean getEnableDownloadLink()
	{
		return enableDownloadLink;
	}
	
	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}
	public String getTooltip()
	{
		return tooltip;
	}
	
	public void setNulltext(String nullText)
	{
		this.nullText = nullText;
	}
	
	public String getNulltext()
	{
		return nullText;
	}
	
	public void setEnableEditLink(boolean enableEditLink)
	{
		this.enableEditLink = enableEditLink;
	}
	public boolean getEnableEditLink()
	{
		return enableEditLink;
	}
	private String ROOT;
	public String getRoot()
	{
		if (ROOT == null) 
		{
			ROOT = ((HttpServletRequest)pageContext.getRequest()).getContextPath()+"/";
		}
		return ROOT;
	}
	
	private PropertyPath getPropertyPath(Object item, String path)
	{
		return Metadata.getInstance(item.getClass()).getPropertyPath(getProperty(), false);
	}
	
	@SuppressWarnings("unchecked")
	public int doStartTag()
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		try
		{
			Object item = getItem();
			Object value = null;	
			DbControl dc = getDbcontrol();
			String ID = dc.getSessionControl().getId();			
			
			if (item != null)
			{
				PropertyPath propertyPath = getPropertyPath(item, getProperty());
				value = propertyPath.getValue(dc, item);
			}
			if (value instanceof Removable)
			{
				if (((Removable)value).isRemoved())
				{
					sb.append("<img class=\"icon\" src=\"").append(getRoot());
					sb.append("images/deleted.gif\" alt=\"This item has been scheduled for deletion\" border=\"0\">");
				}
			}
			
			if (value == null)
			{
				sb.append(getNulltext());
			}
			else if (value instanceof File)
			{
				File file = (File)value;
				if (getEnablePropertyLink())
				{
					sb.append(Base.getLinkedFile(ID, file, false, getEnableEditLink(), getEnableDownloadLink(), getRoot()));
				}
				else
				{
					sb.append(HTML.encodeTags(file.getName()));
				}
			}
			else if (value instanceof Directory)
			{
				Directory d = (Directory)value;
				if (getEnablePropertyLink())
				{
					sb.append(Base.getLink(ID, d.getPath().toString(), Item.DIRECTORY, d.getId(), getEnableEditLink()));
				}
				else
				{
					sb.append(HTML.encodeTags(d.getPath().toString()));
				}
			}
			else if (value instanceof Nameable)
			{
				Nameable pItem = (Nameable)value;
				if (getEnablePropertyLink())
				{		
					sb.append(Base.getLinkedName(ID, pItem, false, getEnableEditLink()));
				}
				else 
				{
					sb.append(HTML.encodeTags(pItem.getName()));				
				}
			}
			else
			{
				sb.append(HTML.encodeTags(value.toString()));
			}
		}
		catch (PermissionDeniedException ex)
		{
			sb.append("<i>- denied -</i>");
		}
		catch (Throwable ex)
		{
			sb.append("<i><span title=\"").append(HTML.encodeTags(ex.getClass().getName()));
			sb.append(": ").append(ex.getMessage()).append("\">- error- </span></i>");
			ex.printStackTrace();
		}
		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return SKIP_BODY;
	}

}




