/*
	$Id: TabControl.java 5322 2010-04-22 07:40:54Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.taglib.tab;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspTagException;
import javax.servlet.jsp.tagext.TagSupport;

/**
	This is the main tag for the tab control. It must always be present as the
	outermost tag. Use one or more {@link Tab} tags to define each tab inside the
	tabcontrol.
	<p>
	<b>Syntax:</b>
	<pre>
   &lt;t:tabcontrol
      id=....
      style=...
      contentstyle=...
      width=...
      active=...
      switch=...
      position=...
      autoheight=true|false
      notabs=true|false
   &gt;
</pre>

	<p>
	<table border=1 cellspacing=0 cellpadding=2>
	<tr>
		<td><b>Attribute</b></td>
		<td><b>Default value</b></td>
		<td><b>Required</b></td>
		<td><b>Description</b></td>
	</tr>
	<tr>
		<td>id</td>
		<td>-</td>
		<td>yes</td>
		<td>The ID of the tab control. The value of this attribute
		goes directly into the standard <code>id</code> attribute.
		This means that if you need a reference to the tab control object in your own
		JavaScript code, you may for example use the following code:
		<p>
		<code>
		tabcontrol = document.getElementById('&lt;id&gt;');
		</code>
		</td>
	</tr>
	<tr>
		<td>style</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute. This allows you to apply different styles
			to different tabcontrols.
		</td>
	</tr>
	<tr>
		<td>contentstyle</td>
		<td>-</td>
		<td>no</td>
		<td>
			The value if this attribute goes directly into the standard HTML 
			<code>style</code> attribute for the content section. This allows you to apply 
			different styles to different tabcontrols.
		</td>
	</tr>
	<tr>
		<td>width</td>
		<td>-</td>
		<td>no</td>
		<td>The width of the tab control.</td>
	</tr>
	<tr>
		<td>active</td>
		<td>-</td>
		<td>no</td>
		<td>
			The id of the tab that should be active to begin with. If no value is
			specified the first tab is automatically made active.
		</td>
	</tr>
	<tr>
		<td>switch</td>
		<td>setActiveTab</td>
		<td>no</td>
		<td>
			The name of the javascript method to call to switch tabs. The function
			should take two parameters which is the ID of the tabcontrol and the ID of the
			tab that should become active. Note! If validation is required the javascript method 
			should call the <code>validateActiveTab(tabControlId)</code>
			method. It is not done automatically.
		</td>
	</tr>
	<tr>
		<td>position</td>
		<td>top</td>
		<td>no</td>
		<td>The position of the tabs in relation to the content. The default value is to display
		the tabs at the top. The only other allowed value is <code>bottom</code>.
		</td>
	</tr>
	<tr>
		<td>autoheight</td>
		<td>true</td>
		<td>no</td>
		<td>
			If the tab control should try to automatically adjust the height of the tab content
			area when the window is resized. This functionality replaces the 
			<code>window.onresize</code> event handler. If an existing event handler exists
			it will be saved as <code>window.oldOnresize</code> and invoked before the 
			tabs are resized.
		</td>
	</tr>
	<tr>
		<td>notabs</td>
		<td>false</td>
		<td>no</td>
		<td>
			If the tabcontrol should display tabs or not. If false the tab control will
			not output any HTML except the contents of the active tab.
		</td>
	</tr>
	</table>

	@author Nicklas
	@version 2.0
	@see Tab
*/
public class TabControl
	extends TagSupport
{
	/**
		
	*/
	private static final long serialVersionUID = 1898616270560551262L;

	/**
		Optional <code>style</code> attribute
	*/
	private String style = null;

	/**
		Optional <code>style</code> attribute for the content section
	*/
	private String contentStyle = null;
	
	/**
		The width of the tab control.
	*/
	private String width = null;
	
	/**
		The ID attribute of the active tab.
	*/
	private String active = null;
	
	/**
		The JavaScript method to call to change tabs.
	*/
	private String switchtab = "TabControl.setActiveTab";
	
	/**
		The position of the tabs.
	*/
	private int position = TOP;
	
	/**
	 	If it should try to remember the last active tab on the page
	*/
	private boolean remember = true;
	
	/**
		If tab height should be automatically adjusted or not.
	*/
	private boolean autoAdjustHeight = true;
	
	/**
		If the tabs should be displayed or not.
	*/
	private boolean noTabs = false;
	
	/**
		The real ID of the tab that is initially active.
	*/
	private String startActive = null;
	
	/**
		The number of tabs.
	*/
	private int numTabs = 0;
	
	/**
		The HTML for the tabs.
	*/
	private StringBuilder tabs;
	
	/**
		The HTML for the contents of the tabs.
	*/
	private StringBuilder content;
	
	/**
		JavaScript for initialising the tabcontrol
	*/
	private StringBuilder script;

	/**
		Tabs are at the top of the tab control.
	*/
	public static final int TOP = 1;
	
	/**
		Tabs are at the bottom of the tab control.
	*/
	public static final int BOTTOM = 2;

	/*
		Taglib initialization methods
	*/
	public void setStyle(String style)
	{
		this.style = style;
	}
	public String getStyle()
	{
		return style;
	}
	
	public void setContentstyle(String contentStyle)
	{
		this.contentStyle = contentStyle;
	}
	public String getContentstyle()
	{
		return contentStyle;
	}

	public void setWidth(String width)
	{
		this.width = width;
	}
	public String getWidth()
	{
		return width;
	}

	public void setActive(String active)
	{
		this.active = active;
	}
	public String getActive()
	{
		return active;
	}
	
	public void setSwitch(String s)
	{
		this.switchtab = s;
	}
	public String getSwitch()
	{
		return switchtab;
	}
	
	public void setPosition(String position)
	{
		if ("bottom".equalsIgnoreCase(position))
		{
			this.position = BOTTOM;
		}
		else
		{
			this.position = TOP;
		}
	}

	public void setAutoheight(boolean auto)
	{
		this.autoAdjustHeight = auto;
	}
	
	public void setNotabs(boolean noTabs)
	{
		this.noTabs = noTabs;
	}
	
	public void setRemember(boolean remember)
	{
		this.remember = remember;
	}
	
	void addTab(Tab tab, String tabcontent)
	{
		numTabs++;
		if (((active == null) && (numTabs == 1)) || (active != null) && active.equals(tab.getId()))
		{
			startActive = tab.getId();
		}

		if (!noTabs)
		{
			String className = position == BOTTOM ? "tab_bottom" : "tab";
	
			tabs.append("<td id=\"").append(getId()).append(".").append(tab.getId()).append("\"");
			tabs.append(" class=\"").append(className).append("\"");
			tabs.append(" onmouseover=\"TabControl.mouseEnter('").append(getId()).append("','").append(tab.getId()).append("')\"");
			tabs.append(" onmouseout=\"TabControl.mouseLeave('").append(getId()).append("','").append(tab.getId()).append("')\"");
			tabs.append(" onclick=\"").append(getSwitch()).append("('").append(getId()).append("','").append(tab.getId()).append("')\"");
			if (tab.getTooltip() != null) tabs.append(" title=\"").append(tab.getTooltip()).append("\"");
			tabs.append(">").append(tab.getTitle()).append("</td>\n");
	
			script.append("TabControl.initTab('");
			script.append(getId()).append("', '").append(tab.getId()).append("', ");
			script.append(tab.getValidate() == null ? "null" : "'"+tab.getValidate()+"'");
			script.append(", ").append(tab.getHelpid() == null ? "null" : "'" + tab.getHelpid() + "'");
			script.append(");\n");
		}
		
		if (tabcontent != null)
		{
			content.append("<div id=\"").append(getId()).append(".").append(tab.getId()).append(".content\"");
 			if (!noTabs) content.append(" style=\"display:none;");
			content.append("\">");
			content.append(tabcontent);
			content.append("</div>\n");
		}

	}

	public int doStartTag()
		throws JspException
	{
		tabs = new StringBuilder();
		content = new StringBuilder();
		script = new StringBuilder();

		if (noTabs) return EVAL_BODY_INCLUDE;
		StringBuilder sb = new StringBuilder();
		sb.append("<table class=\"").append(position == BOTTOM ? "tabcontrol_bottom" : "tabcontrol").append("\"");
		sb.append(" id=\"").append(getId()).append("\"");
		sb.append(" border=0 cellspacing=0 cellpadding=0");
		if (getStyle() != null) sb.append(" style=\"").append(getStyle()).append("\"");
		if (getWidth() != null) sb.append(" width=\"").append(getWidth()).append("\"");
		sb.append(">\n");

		tabs.append("<tr valign=top>\n");
		tabs.append("<td class=\"pretabs\">\n");
		tabs.append("<table border=0 cellspacing=0 cellpadding=0 class=\"").append(position == BOTTOM ? "tabs_bottom" : "tabs").append("\">\n");
		tabs.append("<tr>\n");

		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_BODY_INCLUDE;
	}

	public int doEndTag() 
		throws JspException
	{
		StringBuilder sb = new StringBuilder();
		if (noTabs)
		{
			sb.append(content);
		}
		else
		{
			tabs.append("</tr>\n");
			tabs.append("</table>\n");
			tabs.append("</td>\n");
			tabs.append("<td class=\"").append(position == BOTTOM ? "tabfill_bottom" : "tabfill").append("\">&nbsp;</td>\n");
			tabs.append("</tr>\n");
	
			if (!noTabs && position != BOTTOM) sb.append(tabs);
			if (content.length() > 0)
			{
				sb.append("<tr valign=top>\n");
				sb.append("<td colspan=\"2\">");
				sb.append("<div id=\"").append(getId()).append(".content\"");
	 			sb.append(" class=\"").append(position == BOTTOM ? "tabcontents_bottom" : "tabcontents").append("\"");
	 			if (getContentstyle() != null) sb.append("style=\"").append(getContentstyle()).append("\"");
				sb.append(">");
				sb.append(content);
				sb.append("</div>");
				sb.append("</td>\n</tr>\n");
			}
			if (!noTabs && position == BOTTOM) sb.append(tabs);
	
			sb.append("</table>\n");
			sb.append("<script language=\"JavaScript\">\n");
			sb.append("Main.onLoad(function() {\n");
			sb.append("\tvar switchFunction = TabControl.setActiveTab;\n");
			sb.append("\tif (typeof ").append(getSwitch()).append(" != 'undefined') switchFunction = ").append(getSwitch()).append(";\n");
			sb.append("\tTabControl.setInitialTab('").append(getId()).append("', '").append(startActive).append("', ");
			sb.append(remember ? "1" : "0").append(", switchFunction, ");
			sb.append(autoAdjustHeight ? "true" : "false").append(");\n");
			sb.append(script);
			sb.append("});\n");
			sb.append("</script>\n");
		}
		numTabs = 0;

		try
		{
			pageContext.getOut().print(sb.toString());
		}
		catch (Exception ex)
		{
			throw new JspTagException(ex.getMessage());
		}
		return EVAL_PAGE;
	}
}


