/**
	$Id: AjaxWriter.java 4967 2009-06-10 06:53:35Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.util;

import java.io.PrintWriter;
import java.io.Writer;
import java.util.regex.Pattern;

/**
	Wraps a writer stream and provides methods for easier writing
	of Ajax response. The writer can be configured with 
	{@link #setNullValue(String)} (default is an empty string)
	and {@link #setRecordSeparator(String)} (default is an empty string).
	Use {@link #ajaxPrintEntry(String, Object)} to print a singe line
	with data and {@link #ajaxEndRecord()} to end the current record 
	and start a new one.

	@author Nicklas
	@version 2.12
	@base.modified $Date: 2009-06-10 08:53:35 +0200 (Wed, 10 Jun 2009) $
*/
public class AjaxWriter
	extends PrintWriter
{

	private String nullValue = "";
	private String recordSeparator = "";

	/**
		Create a new ajax writer that is writing it's output to the
		given writer.
		@param out The writer to write the data to
	*/
	public AjaxWriter(Writer out)
	{
		super(out);
	}
	
	/**
		The null value string is written to the output instead of
		null data values. The default value is an empty string.
		@return The current value of the null value string
	*/
	public String getNullValue()
	{
		return nullValue;
	}
	
	/**
		Change the null value string.
		@param nullValue The new null value string
	*/
	public void setNullValue(String nullValue)
	{
		this.nullValue = nullValue;
	}

	/**
		The record separator is written to the output on a single
		line between each record. The default value is an empty string.
		@return The current value of the record separator
	*/
	public String getRecordSeparator()
	{
		return recordSeparator;
	}
	
	/**
		Change the record separator string.
		@param recordSeparator The new record separator
	*/
	public void setRecordSeparator(String recordSeparator)
	{
		this.recordSeparator = recordSeparator;
	}

	/**
		Print a record entry. Line breaks (\n) and
		carriage returns (\r) in the value are converted to
		the literal string values '\n' and '\r'.
		@param key The entry key
		@param value The value, if null the configured null value
			is used, otherwise the toString() method is called
	*/
	public void ajaxPrintEntry(String key, Object value)
	{
		String valueToWrite = value == null ? getNullValue() : value.toString();
		print(key + ":" + ajaxEscape(valueToWrite)+"\n");
	}
	
	/**
		End the current record by printing the record separator
		string on a line by its own. The default is to print an
		empty line.
	*/
	public void ajaxEndRecord()
	{
		print(getRecordSeparator()+"\n");
	}
	
	
	private Pattern r = Pattern.compile("\r");
	private Pattern n = Pattern.compile("\n");

	/**
		Escape new line and carrige return characters in the input
		to the literal string '\n' and '\r' in the output.
		@param in The input string
	*/
	public String ajaxEscape(String in)
	{
		in = r.matcher(in).replaceAll("\\\\r");
		in = n.matcher(in).replaceAll("\\\\n");
		return in;
	}


}
