/**
	$Id: BioPlate.java 5456 2010-10-28 12:12:26Z nicklas $

	Copyright (C) 2008 Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.BioPlateData;
import net.sf.basedb.core.data.BioWellData;
import net.sf.basedb.core.data.HardwareData;
import net.sf.basedb.core.data.PlateCoordinate;
import net.sf.basedb.core.query.Hql;
import net.sf.basedb.core.query.Orders;
import net.sf.basedb.core.query.Restrictions;

import java.util.List;
import java.util.Set;

/**
 	This class is represents physical bio-plates. A bio-plate must have
	a {@link PlateGeometry}
	<p>
	A bio-plate consists of {@link BioWell}:s which are organised into rows and
	columns as specified by the plate geometry.	
  	@author martin
  	@version 2.10
  	@base.modified $Date: 2010-10-28 14:12:26 +0200 (Thu, 28 Oct 2010) $
 	
 */
public class BioPlate 
	extends AnnotatedItem<BioPlateData>
{
	/**
		The type of item represented by this class.
		@see Item#BIOPLATE
		@see #getType()
	*/
	public static final Item TYPE = Item.BIOPLATE;

	/**
		The maximum length of the barcode that can be stored in the
		database. Check the length against this value before calling the
		{@link #setBarcode(String)} method to avoid exceptions.
	*/
	public static final int MAX_BARCODE_LENGTH = BioPlateData.MAX_BARCODE_LENGTH;
	
	/**
	  	@deprecated In 2.16, use {@link #getNew(DbControl, PlateGeometry, BioPlateType)} instead
	  	Will try to find a bioplate type that is not locked to a specific biomaterial
	  	type. If not found an exception is thrown.
	*/
	public static BioPlate getNew(DbControl dc, PlateGeometry plateGeometry)
		throws BaseException, InvalidDataException
	{
		ItemQuery<BioPlateType> query = BioPlateType.getQuery();
		query.include(Include.ALL);
		query.restrict(Restrictions.eq(Hql.property("bioMaterialType"), null));
		query.order(Orders.asc(Hql.property("id")));
		List<BioPlateType> result = query.list(dc);
		if (result.size() == 0) throw new ItemNotFoundException("BioPlateType");
		return getNew(dc, plateGeometry, result.get(0));
	}
	
	/**
		Creates a new bioplate with a specific plate geometry and type.
	  	@param dc DbControl used for database access
	  	@param geometry The geometry the new plate should have 
	  	@param plateType The type of the new plate
	  	@return A new BioPlate object
	  	@throws BaseException If anything goes wrong when creating.
	  	@throws InvalidDataException If gemoetry or type is null
	  	@since 2.16
	*/
	public static BioPlate getNew(DbControl dc, PlateGeometry geometry, BioPlateType plateType)
		throws BaseException, InvalidDataException
	{
		BioPlate bp = dc.newItem(BioPlate.class);
		bp.setPlateGeometry(geometry);
		bp.setBioPlateType(plateType);
		bp.setName("New bioplate");		
		return bp;
	}

	
	/**
	 	Gets a BioPlate when knowing the item's id
	  	@param dc DbControl used for database access
	  	@param id The item's id.
	  	@return <code>BioPlate</code> object
	  	@throws ItemNotFoundException If no item with the given id was found
	  	@throws PermissionDeniedException If logged in user doesn't have enough 
	  		permission to the requested item
	  	@throws BaseException If there is another error.
	 */
	public static BioPlate getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		BioPlate p = dc.loadItem(BioPlate.class, id);
		if (p == null) throw new ItemNotFoundException("BioPlate[id="+id+"]");
		return p;
	}
	
	/**
	 	Gets query that selects all available bioplates
	  	@return <code>ItemQuery</code> object
	 */
	public static ItemQuery<BioPlate> getQuery()
	{
		return new ItemQuery<BioPlate>(BioPlate.class);
	}

	BioPlate(BioPlateData annotatedData)
	{
		super(annotatedData);
	}
	
	/**
	 	Gets parent plates, always returns null
	 */
	@Override
	public Set<Annotatable> getAnnotatableParents() 
		throws BaseException
	{
		return null;
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	@Override
	public Item getType()
	{
		return TYPE;
	}	
	//-------------------------------------------
	
	/*
	 	From the BasicItem class
	 	--------------------------------------
	 */
	/**
	 	Check if there are any <code>MeasuredBioMaterial</code> on
	 	this plate.
	 */
	@Override
	public boolean isUsed()
	{
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOMATERIAL_ON_PLATE", "count(*)");
		/*
		 	SELECT {1}
		 	FROM MeasuredBioMaterialData mbm
		 	JOIN mbm.bioWell bw
		 	WHERE bw.bioPlate = :bioPlate
		 */
		query.setEntity("bioPlate", this.getData());
		return HibernateUtil.loadData(Long.class, query) > 0 || super.isUsed();
	}
	
	/**
	 	Gets all <code>MeasuredBioMaterial</code> located on this plate
	 	and all child plates created from this plate.
	 */
	@Override
	public Set<ItemProxy> getUsingItems()
	{
		Set<ItemProxy> using = super.getUsingItems();
		org.hibernate.Session session = getDbControl().getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, 
				"GET_BIOMATERIAL_ON_PLATE", "mbm");
		/*
		 	SELECT {1}
		 	FROM MeasuredBioMaterialData mbm
		 	JOIN mbm.BioWell bw
		 	WHERE bw.bioPlate = :bioPlate
		 */
		query.setEntity("bioPlate", this.getData());
		addUsingItems(using, query);
		return using;
	}
	//-------------------------------------------

	/**
		Get the {@link PlateGeometry} of this bioplate.
		@return The <code>PlateGeometry</code> item
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#READ} permission for the plate geometry
		@throws BaseException If there is another error
	*/
	public PlateGeometry getPlateGeometry()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(PlateGeometry.class, getData().getPlateGeometry()); 
	}
	
	private void setPlateGeometry(PlateGeometry plateGeometry)
		throws PermissionDeniedException, InvalidDataException
	{
		if (plateGeometry == null) throw new InvalidUseOfNullException("plateGeometry");
		plateGeometry.checkPermission(Permission.USE);
		getData().setPlateGeometry(plateGeometry.getData());		
	}

	/**
		Get the {@link BioPlateType} of this bioplate.
		@return The <code>BioPlateType</code> item
		@throws PermissionDeniedException If the logged in user doesn't have
			{@link Permission#READ} permission for the plate type
		@throws BaseException If there is another error
		@since 2.16
	*/
	public BioPlateType getBioPlateType()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(BioPlateType.class, getData().getBioPlateType()); 
	}
	
	private void setBioPlateType(BioPlateType plateType)
		throws PermissionDeniedException, InvalidDataException
	{
		if (plateType == null) throw new InvalidUseOfNullException("plateType");
		plateType.checkPermission(Permission.USE);
		getData().setBioPlateType(plateType.getData());		
	}

	
	/**
	 	Sets the external id of this bio plate
	  	@param externalId The external id to set. Null is allowed.
	  		Maximum length {@link BioPlateData#MAX_EXTERNAL_ID_LENGTH} characters.
	  	@throws PermissionDeniedException If logged in user doesn't have 
	  		{@link Permission#WRITE} on the plate
	  	@throws InvalidDataException If maximum length of externalId is exceeded.
	 */
	public void setExternalId(String externalId)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExternalId(StringUtil.setNullableString(externalId, "externalId", BioPlateData.MAX_EXTERNAL_ID_LENGTH));
	}
	
	/**
	 	Gets the external id of this bio plate
	  	@return A String object or null
	 */
	public String getExternalId()
	{
		return getData().getExternalId();
	}
	
	/**
	 	Sets the bio plates barcode
	  	@param barcode The barcode to set. Null is allowed.
	  		Maximum length {@link BioPlateData#MAX_BARCODE_LENGTH} characters.
	  	@throws PermissionDeniedException If logged in user doesn't have 
	  		{@link Permission#WRITE} on the plate.
	  	@throws InvalidDataException If maximum length of barcode is exceeded.
	 */
	public void setBarcode(String barcode)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setBarcode(StringUtil.setNullableString(barcode, "barcode", BioPlateData.MAX_BARCODE_LENGTH));
	}
	
	/**
	 	Gets the barcode of this bio plate.
	  	@return A String object or null.
	 */
	public String getBarcode()
	{
		return getData().getBarcode();
	}
	
	/**
		Check if the plate has been destroyed. This is currently only
		a flag. It doesn't limit what can be done with the plate through
		the API.
		@since 2.16
	*/
	public boolean isDestroyed()
	{
		return getData().isDestroyed();
	}
	
	/**
		Set the destroyed flag. 
	 	@param destroyed TRUE if the plate is destroyed, FALSE otherwise. 
	 	@throws PermissionDeniedException If the logged in user doesn't 
	 		have write permission on the item.
	 	@since 2.16
	*/
	public void setDestroyed(boolean destroyed)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setDestroyed(destroyed);
	}
	
	/**
	 	Sets the freezer where this bio plate is stored
	  	@param freezer The freezer to set. Null is allowed.
	  	@throws PermissionDeniedException If logged in user doesn't have
	  		{@link Permission#WRITE} on this bio plate or 
	  		{@link Permission#USE} on the freezer.
	 */
	public void setFreezer(Hardware freezer)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		HardwareData hw = null;
		if (freezer != null)
		{
			freezer.checkPermission(Permission.USE);
			hw = freezer.getData();
		}
		getData().setFreezer(hw);
	}
	
	/**
	 	Gets the freezer associated with this bio plate
	  	@return A {@link Hardware} item or null.
	 */
	public Hardware getFreezer()
	{
		return getDbControl().getItem(Hardware.class, getData().getFreezer());
	}
	
	/**
	 	Gets a query that returns all <code>BioWell</code>s 
	 	on this plate.
	  	@return an {@link ItemQuery} object.
	 */
	public ItemQuery<BioWell> getBioWells()
	{
		return BioWell.getQuery(this);
	}

	/**
	 	Gets bio well for specific location.
	  	@param row Row index where the bio well is located.
	  	@param column Column index where the bio well is located
	  	@return A {@link BioWell} object or 
	  		null if no bio well can be found
	 */
	public BioWell getBioWell(int row, int column)
	{
		BioWellData wellData = getData().getBioWells().get(new PlateCoordinate(row, column));
		return getDbControl().getItem(BioWell.class, wellData);
	}
	
	/**
	 	Gets bio well for specific location.
	  	@param wellC The coordinate of the well
	  	@return A {@link BioWell} object or 
	  		null if no bio well can be found
	  	@since 2.16
	 */
	public BioWell getBioWell(PlateCoordinate wellC)
	{
		if (wellC == null) return null;
		BioWellData wellData = getData().getBioWells().get(wellC);
		return getDbControl().getItem(BioWell.class, wellData);
	}
	
	/**
		Get the biomaterial in the well at the given location. This method
		is a shortcut for BioPlate.getBioWell().getBioMaterial().
	  	@param wellC The coordinate of the well
		@return A MeasuredBioMaterial or null if the well doesn't exists 
			or is empty
		@since 2.16
	 */
	public MeasuredBioMaterial getBioMaterial(PlateCoordinate wellC)
	{
		if (wellC == null) return null;
		BioWellData well = getData().getBioWells().get(wellC);
		if (well == null) return null;
		return getDbControl().getItem(MeasuredBioMaterial.class, well.getBioMaterial());
	}
	

}