/*
	$Id: DataCubeExtraValue.java 4889 2009-04-06 12:52:39Z nicklas $
	
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.DataCubeExtraValueData;
import net.sf.basedb.core.data.SharedData;

/**
	Represents a set of extra value in a data cube. The value type and
	coordinate type determines the database table where the extra values
	are stored. This item keeps track of the number of bytes the extra values
	occupies on disk. The number of bytes is reported up to the {@link Experiment} 
	which is the item that handles user quota. 

	@author Nicklas
	@version 2.0
	@see ExtraValue
	@see ExtraValueType
	@see SpotExtraValueBatcher
	@see PositionExtraValueBatcher
	@see DataCube
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class DataCubeExtraValue
	extends ChildItem<DataCubeExtraValueData>
{
	/**
		The type of item represented by this class.
		@see Item#DATACUBEEXTRAVALUE
		@see #getType()
	*/
	public static final Item TYPE = Item.DATACUBEEXTRAVALUE;

	/**
		Create a new <code>DataCubeExtraValue</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access.
		@param cube The cube the new extra value is located in
		@param valueType The data type of the extra values
		@param coordinateType Which coordinates the extra values should be attached to
		@return The new <code>DataCubeExtraValue</code> item
		@throws BaseException If there is an error
	*/
	static DataCubeExtraValue getNew(DbControl dc, DataCube cube, 
		Type valueType, ExtraValue.CoordinateType coordinateType)
		throws BaseException
	{
		DataCubeExtraValue ev = dc.newItem(DataCubeExtraValue.class);
		ev.setDataCube(cube);
		ev.setValueType(valueType);
		ev.setCoordinateType(coordinateType);
		ev.setExtraNo(cube.getNextExtraValueNo());
		return ev;
	}

	/**
		Get a <code>DataCubeExtraValue</code> item when you know the id.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The id of the item to load
		@return The <code>DataCubeExtraValue</code> item
		@throws ItemNotFoundException If an item with the specified id 
			is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the item
		@throws BaseException If there is another error
	*/
	public static DataCubeExtraValue getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		DataCubeExtraValue ev = dc.loadItem(DataCubeExtraValue.class, id);
		if (ev == null) throw new ItemNotFoundException("DataCubeExtraValue[id=" + id + "]");
		return ev;
	}


	/**
	 	Creates a new data cube item from the given data.
		@param data the data
	*/
	DataCubeExtraValue(DataCubeExtraValueData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------

	/*
		From BasicItem
		-------------------------------------------
	*/
	/**
		Delete data and update disk usage information on the 
		experiment if action == Action.DELETE
	*/
	void onBeforeCommit(Transactional.Action action)
		throws BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.DELETE)
		{
			getDataCube().addBytes(-getBytes());
			getVirtualDb().deleteAllCubeExtraValueData(getDataCubeNo(), getExtraNo(), getVirtualTable());
		}
	}
	// -------------------------------------------
	/*
		From the ChildItem class
		-------------------------------------------
	*/
	/**
		Get the experiment.
	*/
	SharedData getSharedParent()
	{
		return getData().getDataCube().getVirtualDb().getExperiment();
	}
	/**
		USE permission is enough to be able to manage data cubes.
	*/
	Permission getPermissionForWriteDeleteAndCreate()
	{
		return Permission.USE;
	}
	// -------------------------------------------

	/**
		Get the data cube this extra value belongs to.
		@return A <code>DataCube</code> object
		@throws PermissionDeniedException If the logged in user doesn't have 
			read permission to the item
		@throws BaseException If there is another error
	*/
	public DataCube getDataCube()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(DataCube.class, getData().getDataCube());
	}
	
	/**
		Set the data cube. This cannot be changed after the extra value 
		has been created.
	*/
	private void setDataCube(DataCube cube)
	{
		getData().setDataCube(cube.getData());
	}

	/**
		Get the coordinate type the extra values are attached to.
	*/
	public ExtraValue.CoordinateType getCoordinateType()
	{
		return ExtraValue.CoordinateType.fromValue(getData().getCoordinateType());
	}
	
	/**
		Set the coordinate type. Can't be changed once it has been saved.
	*/
	private void setCoordinateType(ExtraValue.CoordinateType coordinateType)
		throws InvalidDataException
	{
		if (coordinateType == null) throw new InvalidUseOfNullException("coordinateType");
		getData().setCoordinateType(coordinateType.getValue());
	}
	
	/**
		Get the value type of the extra values.
	*/
	public Type getValueType()
	{
		return Type.fromValue(getData().getValueType());
	}
	
	/**
		Set the value type. Can't be changed once it has been saved.
	*/
	private void setValueType(Type valueType)
		throws InvalidDataException
	{
		if (valueType == null) throw new InvalidUseOfNullException("valueType");
		getData().setValueType(valueType.getValue());
	}

	/**
		Get the number of the data cube this extra value belongs to.
	*/
	public short getDataCubeNo()
	{
		return getData().getDataCube().getCubeNo();
	}

	/**
		Get the extra value number.
	*/
	public short getExtraNo()
	{
		return getData().getExtraNo();
	}
	private void setExtraNo(short extraNo)
	{
		getData().setExtraNo(extraNo);
	}
	
	/**
		Get the number of bytes the data stored in this layer occupies on disk.
	*/
	public long getBytes()
	{
		return getData().getBytes();
	}
	void addBytes(long bytes)
	{
		getData().setBytes(bytes + getData().getBytes());
		getDataCube().addBytes(bytes);
	}

	/**
		Get the virtual database this extra value belongs to.
	*/
	VirtualDb getVirtualDb()
	{
		return getDbControl().getItem(VirtualDb.class, getData().getDataCube().getVirtualDb());
	}
	
	/**
		Get the virtual table this extra value stores it's data in.
	*/
	VirtualTable getVirtualTable()
	{
		return getCoordinateType().getVirtualTable(getValueType());
	}
	
}
