/*
  $Id: MimeType.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Johan Enell, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import java.util.Date;

import net.sf.basedb.core.data.MimeTypeData;
/**
	This class is used to represent mime types. The mime types are important
	for file handling, for example when downloading a file on the web.
	
	@author enell
	@version 2.0
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public class MimeType
	extends BasicItem<MimeTypeData>
	implements Nameable, Removable, Registered
{
	
	/**
		The type of item represented by this class.
		@see Item#MIMETYPE
		@see #getType()
	*/
	public static final Item TYPE = Item.MIMETYPE;

	/**
		The maximum length of the extension variable that can be stored in the
		database. Check the length against this value before calling the
		{@link #setExtension(String)} method to avoid
		exceptions.
	*/
	public static final int MAX_EXTENSION_LENGTH = MimeTypeData.MAX_EXTENSION_LENGTH;
	
	/**
		Create a new <code>MimeType</code> item.

		@param dc The {@link DbControl} which will be used for
			permission checking and database access
		@return The new <code>MimeType</code> item
		@throws BaseException If there is an error
	*/
	public static MimeType getNew(DbControl dc)
		throws BaseException
	{
		MimeType mimeType = dc.newItem(MimeType.class);
		mimeType.setName("New MIME type");
		mimeType.setExtension("new");
		mimeType.getData().setEntryDate(new Date());
		return mimeType;
	}

	/**
		Get a <code>MimeType</code> object when you know the ID.

		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param id The ID of the item to load
		@return The <code>MimeType</code> item
		@throws ItemNotFoundException If an item with the specified ID is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the items
		@throws BaseException If there is another error
	*/
	public static MimeType getById(DbControl dc, int id)
		throws ItemNotFoundException, PermissionDeniedException, BaseException
	{
		MimeType mt = dc.loadItem(MimeType.class, id);
		if (mt == null) throw new ItemNotFoundException("MimeType[id="+id+"]");
		return mt;
	}

	/**
		Get a <code>MimeType</code> item when you know the file extension.
	
		@param dc The <code>DbControl</code> which will be used for
			permission checking and database access.
		@param extension The extension of the mime type, do not include the
			dot in the extension
		@return The <code>MimeType</code> item
		@throws ItemNotFoundException If an item with the specified ID is not found
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#READ} permission to the items
		@throws BaseException If there is another error
	*/
	public static MimeType getByExtension(DbControl dc, String extension)
		throws PermissionDeniedException, ItemNotFoundException, BaseException
	{
		org.hibernate.Session session = dc.getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, "GET_MIME_TYPE_FROM_EXTENSION");
		/*
			SELECT mt
			FROM mimetype mt
			WHERE mt.extension = :extension
		*/
		query.setString("extension", extension);
		MimeType mt = dc.getItem(MimeType.class, HibernateUtil.loadData(MimeTypeData.class, query));
		if (mt == null) throw new ItemNotFoundException("MimeType[extension="+extension+"]");
		return mt;
	}
	
	/**
		Get a query configured to retrieve MIME types.
		@return An {@link ItemQuery} object
	 	@throws BaseException If the query could not be created. 
	*/
	public static ItemQuery<MimeType> getQuery()
		throws BaseException
	{
		return new ItemQuery<MimeType>(MimeType.class);
	}
		
	/**
		Get the MIME type of a given file name or extension. This method will check 
		all registered MIME types even if the logged in user doesn't have read permission.
		@param dc The DbControl to use for database access
		@param name The file name or extension, if a dot exists in the name only
			the last part will be used for MIME type lookup
		@param defaultMimeType A default MIME type to return if none is found
		@return A MIME type or null
		@throws BaseException If there is an error
		@since 2.2
	 */
	public static String getMimeType(DbControl dc, String name, String defaultMimeType)
		throws BaseException
	{
		int dot = name.lastIndexOf('.');
		String mimeType = defaultMimeType;

		String extension = dot >= 0 ? name.substring(dot + 1) : name;
		org.hibernate.Session session = dc.getHibernateSession();
		org.hibernate.Query query = HibernateUtil.getPredefinedQuery(session, "GET_MIME_TYPE_FROM_EXTENSION");
		/*
			SELECT mt
			FROM mimetype mt
			WHERE mt.extension = :extension
		*/
		query.setString("extension", extension);
		MimeTypeData mt = HibernateUtil.loadData(MimeTypeData.class, query);
		if (mt != null) mimeType = mt.getName();
		return mimeType;
	}
	
	/**
		Creates a new mimetype item from the given data.

		@param data the data.
	*/
	MimeType(MimeTypeData data)
	{
		super(data);
	}

	/*
		From the Identifiable interface
		-------------------------------------------
	*/
	public Item getType()
	{
		return TYPE;
	}
	// -------------------------------------------
	/*
		From the Nameable interface
		-------------------------------------------
	*/
	public String getName()
	{
		return getData().getName();
	}
	public void setName(String name)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setName(getData(), name);
	}
	public String getDescription()
	{
		return getData().getDescription();
	}
	public void setDescription(String description)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		NameableUtil.setDescription(getData(), description);
	}
	// -------------------------------------------
	/*
		From the Removable interface
		-------------------------------------------
	*/
	public boolean isRemoved()
	{
		return getData().isRemoved();
	}
	public void setRemoved(boolean removed)
		throws PermissionDeniedException
	{
		checkPermission(removed ? Permission.DELETE : Permission.WRITE);
		getData().setRemoved(removed);
	}
	// -------------------------------------------
	/*
		From the Registered interface
		-------------------------------------------
	*/
	public Date getEntryDate()
	{
		return DateUtil.copy(getData().getEntryDate());
	}
	// -------------------------------------------

	/**
		Set the extension variable for this <code>MimeType</code> item.
		
		@param extension A string with the extension of this item. Note that this 
			extension should be without leading dot.  
		@throws PermissionDeniedException If the logged in user doesn't have 
			{@link Permission#WRITE} permission to the mimetype.
		@throws InvalidDataException If the new value is null or longer than
			{@link MimeType#MAX_EXTENSION_LENGTH}
	*/
	public void setExtension(String extension) throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.WRITE);
		getData().setExtension(StringUtil.setNotNullString(extension, "extension", MAX_EXTENSION_LENGTH).toLowerCase());
	}
	
	/**
		Get the extension for this <code>MimeType</code> item.
		@return A string with the extension of this item
	*/
	public String getExtension()
	{
		return getData().getExtension();
	}

	/**
		Get the associated {@link FileType} item.
		@return The <code>FileType</code> item, or null if no type has been specified
		@throws PermissionDeniedException This exception is thrown if
			the logged in user doesn't have read
			permission to the item
		@throws BaseException If there is another error
	*/
	public FileType getFileType() 
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(FileType.class, getData().getFileType());
	}
	
	/**
		Set the filetype of this item.
		@param filetype The new <code>FileType</code>, or null
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
	*/
	public void setFileType(FileType filetype)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setFileType(filetype == null ? null : filetype.getData());
	}
	
	/**
		If files of this MIME type should automatically be compressed
		when they are uploaded to BASE. Note! Auto-compression must be
		enabled. See {@link Application#autoCompressionEnabled()}.
		@return TRUE to automatically compress files of this type
		@since 2.5
	*/
	public boolean getAutoCompress()
	{
		return getData().getAutoCompress();
	}
	
	/**
		Set if files of this MIME type should automatically
		be compressed when they are uploaded to BASE. Note! Auto-compression must be
		enabled. See {@link Application#autoCompressionEnabled()}.
		@param autoCompress TRUE to enable auto-compression
		@throws PermissionDeniedException If the logged in user doesn't have
			write permission
		@since 2.5
	*/
	public void setAutoCompress(boolean autoCompress)
		throws PermissionDeniedException
	{
		checkPermission(Permission.WRITE);
		getData().setAutoCompress(autoCompress);
	}

}
