/*
	$Id: OwnedItem.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005 Samuel Andersson, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

import net.sf.basedb.core.data.OwnedData;
import net.sf.basedb.core.data.UserData;
import net.sf.basedb.info.OwnedItemInfo;

/**
	This class inherits from the {@link BasicItem}
	class and implements the {@link Ownable} interface.
	An <code>Ownable</code> item is an item that has a
	{@link User} as its owner.
	<p>

	@author Nicklas
	@version 2.0
	@see Ownable
	@base.modified $Date: 2009-04-06 14:52:39 +0200 (Mon, 06 Apr 2009) $
*/
public abstract class OwnedItem<D extends OwnedData>
	extends BasicItem<D>
	implements Ownable
{
	OwnedItem(D ownedData)
	{
		super(ownedData);
	}

	/*
		From the Ownable interface
		-------------------------------------------
	*/
	public User getOwner()
		throws PermissionDeniedException, BaseException
	{
		return getDbControl().getItem(User.class, getData().getOwner());
	}
	public void setOwner(User owner)
		throws PermissionDeniedException, InvalidDataException
	{
		checkPermission(Permission.SET_OWNER);
		OwnableUtil.setOwner(getData(), owner);
	}
	public void takeOwnership()
		throws PermissionDeniedException
	{
		checkPermission(Permission.SET_OWNER);
		DbControl dc = getDbControl();
		getData().setOwner(HibernateUtil.loadData(
			dc.getHibernateSession(), UserData.class, dc.getSessionControl().getLoggedInUserId()));
	}
	public boolean isOwner()
	{
		return getData().getOwner().getId() == getSessionControl().getLoggedInUserId();
	}
	// -------------------------------------------

	/*
		From the BasicItem class
		-------------------------------------------
	*/
	/**
		Overrides the {@link BasicItem#onBeforeCommit(Transactional.Action)}
		method. It sets the owner of the new item to the logged in user, unless
		it has already been specified.
		@throws NotLoggedInException If no user is logged in
		@throws BaseException If there is another error
	*/
	void onBeforeCommit(Transactional.Action action)
		throws NotLoggedInException, BaseException
	{
		super.onBeforeCommit(action);
		if (action == Transactional.Action.CREATE && getData().getOwner() == null)
		{
			org.hibernate.Session session = getDbControl().getHibernateSession();
			int loggedInuserId = getSessionControl().getLoggedInUserId();
			UserData owner = HibernateUtil.loadData(session, UserData.class, loggedInuserId);
			if (owner == null) throw new NotLoggedInException();
			getData().setOwner(owner);
		}
	}
	/**
		If the logged in user is the owner of this item, DELETE,
		SET_OWNER and SET_PERMISSION permission is granted.
	*/
	void initPermissions(int granted, int denied)
		throws BaseException
	{
		UserData owner = getData().getOwner();
		// owner may be null for new items
		if (owner != null && owner.getId() == getSessionControl().getLoggedInUserId())
		{
			granted |= Permission.grant(Permission.DELETE, Permission.SET_OWNER, Permission.SET_PERMISSION);
		}
		super.initPermissions(granted, denied);
	}
	
	/**
	 	@since 2.5
	 */
	OwnedItemInfo toTransferable(OwnedItemInfo info)
	{
		super.toTransferable(info);
		info.setOwnerId(getData().getOwner().getId());
		return info;
	}
}
