/*
	$Id: ShareableData.java 4889 2009-04-06 12:52:39Z nicklas $

	Copyright (C) 2005, 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

/**
	A shareable item is an item which can be shared to other users, 
	groups or projects. To be able to share an item, it must have an
	owner, thus this interface extends the {@link OwnableData}
	interface. 
	<p>
	Access permissions are held in a {@link ItemKeyData} object
	for users and groups and a {@link ProjectKeyData} object for
	projects.
	<p>
	The {@link SharedData} class provides an implementation
	for this interface and it is recommended that shareable classes inherit
	from that class.
	<p>
	This interface defines Hibernate database mappings for the
	<code>itemKey</code> and <code>projectKey</code> properties to default 
	database columns. If a subclass wants to map these properties to other columns,
	it should override the {@link #getItemKey()} and {@link #getProjectKey()} methods
	and add a Hibernate tag in the comment.
	<p>
	
	<b>Reference implementation</b><br>
	<pre class="code">
private ItemKeyData itemKey;
public ItemKeyData getItemKey()
{
   return itemKey;
}
public void setItemKey(ItemKeyData itemKey)
{
   this.itemKey = itemKey;
}
private ProjectKeyData projectKey;
public ProjectKeyData getProjectKey()
{
   return projectKey;
}
public void setProjectKey(ProjectKeyData projectKey)
{
   this.projectKey = projectKey;
}
</pre>

	@author Nicklas
	@version 2.0
	@see SharedData
	@see net.sf.basedb.core.Shareable
	@see <a href="../../../../../../html/developerdoc/api/data_api.authentication.html">Authentication overview</a>
	@see <a href="../../../../../../html/developerdoc/api/data_api.basic.html">Basic classes and interfaces</a>
*/
public interface ShareableData
	extends OwnableData
{

	/**
		Get the {@link ItemKeyData} for the item. An item key is used
		to share an item to individual users and/or groups.
		@hibernate.many-to-one column="`itemkey_id`" not-null="false" outer-join="false"
	*/
	public ItemKeyData getItemKey();

	/**
		Set the {@link ItemKeyData} for the item. An item key is used
		to share an item to individual users and or groups. Use null to
		disable sharing.
	*/
	public void setItemKey(ItemKeyData key);


	/**
		Get the {@link ProjectKeyData} for the item. A project key is used
		to share an item to projects.
		@hibernate.many-to-one column="`projectkey_id`" not-null="false" outer-join="false"
	*/
	public ProjectKeyData getProjectKey();

	/**
		Set the {@link ProjectKeyData} for the item. A project key is used
		to share an item to projects. Use null to disable sharing.
	*/
	public void setProjectKey(ProjectKeyData key);

}
