/**
	$Id: EmailUtil.java 5442 2010-10-07 06:48:16Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.mail.Email;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.SimpleEmail;

import net.sf.basedb.core.Application;
import net.sf.basedb.core.Config;
import net.sf.basedb.core.User;

/**
	Utility class for email sending. Before use it must be initialized 
	(this is done automatically when starting BASE). Configuration options
	in the base.config file are used.
	
	<ul>
	<li>mail.server.host: Host name of the outgoing smtp server. This
		must be set or email sending will be disabled.
	<li>mail.server.port: Port the outgoing smtp server is listening to.
		If not set the default port is used (25 for regular mail, 465 for
		SSL).
	<li>mail.server.ssl: If SSL is enabled or not.
	<li>mail.server.tsl: If TLS is enabled or not.
	<li>mail.from.email: Email address to use as the sender of outgoing emails.
		This must be set or email seding will be disabled.
	<li>mail.from.name: Name to use as the sender of outgoing emails. If not
		set a default name is created based on the host name of the BASE server.
	</ul>
	
	@author Nicklas
	@since 2.16
	@base.modified $Date: 2010-10-07 08:48:16 +0200 (Thu, 07 Oct 2010) $
*/
public class EmailUtil
{

	/**
		This pattern can be used to find email addresses. The pattern will check that
		an @ symbol is present and that it is preceded with at least one character and
		followed by at least one subdomain and one topdomain. The pattern will allow
		any unicode letters, digits, underscore and hyphen in the address. It will
		not check that the domain or email address actually exists.
	*/
	public static final Pattern EMAIL_REGEXP = Pattern.compile("[\\p{L}\\d_-]+(\\.[\\p{L}\\d_-]+)*@[\\p{L}\\d_-]+(\\.[\\p{L}\\d_-]+)+");

	private static boolean initialized;
	private static boolean enabled;
	
	private static String hostName;
	private static int hostPort;
	private static boolean ssl;
	private static boolean tls;
	private static String senderEmail;
	private static String senderName;
	
	public static synchronized void init()
	{
		if (initialized) return;
		
		hostName = Config.getString("mail.server.host", null);
		ssl = Config.getBoolean("mail.server.ssl");
		tls = Config.getBoolean("mail.server.tls");
		hostPort = Config.getInt("mail.server.port", ssl ? 465 : 25);
		senderEmail = Config.getString("mail.from.email", null);
		senderName = Config.getString("mail.from.name", "BASE " + Application.getMajorVersion() + "." + 
			Application.getMinorVersion() + "." + Application.getMaintenanceVersion() + "@" + Application.getHostName());
		enabled = hostName != null && senderEmail != null;
		initialized = true;
	}
	
	/**
		Unload all settings.
	*/
	public static synchronized void unload()
	{
		initialized = false;
		enabled = false;
		hostName = null;
		ssl = false;
		tls = false;
		hostPort = 0;
		senderEmail = null;
		senderName = null;
	}

	/**
		Check if email sending has been enabled for the server or not. Sending is disabled
		if the {@link #init()} method hasn't been called, the server admin hasn't
		specified an outgoing smtp server or an outgoing sender email.
		@return TRUE if mails are enabled, FALSE otherwise
	*/
	public static boolean isEnabled()
	{
		return initialized && enabled;
	}
	
	/**
		Check if email sending is enabled for the given user. All of the
		following conditions must apply:
		<ul>
		<li>The server must be configured for sending emails (see {@link #isEnabled()}).
		<li>The user must have selected to have emails ({@link User#getSendMessagesAsEmail()}).
		<li>The user must have a valid email address.
		</ul>

		@param user The user to check
	*/
	public static boolean isEnabled(User user)
	{
		return isEnabled() && user.getSendMessagesAsEmail() && isValidEmail(user.getEmail());
	}
	
	/**
		Checks if the given string looks like an email address. This is done
		by trying to match it against the {@link #EMAIL_REGEXP}
		pattern.
	
		@param email The string to check
		@return TRUE or FALSE
		@see #EMAIL_REGEXP
	*/
	public static boolean isValidEmail(String email)
	{
		if (email == null) return false;
		Matcher m = EMAIL_REGEXP.matcher(email);
		return m.matches();
	}
	
	/**
		Create a simple text email. Host name, sender, etc. will be set
		according to the settigs from base.config. The caller need
		to set a 'To' address, 'Subject' and the actual message.
		
		@return An Email object, or null if mail sending is not enabled
		@throws EmailException If there is something wrong with the setup
	*/
	public static Email createSimpleEmail()
		throws EmailException
	{
		if (!isEnabled()) return null;
		Email email = new SimpleEmail();
		email.setHostName(hostName);
		if (ssl)
		{
			email.setSSL(true);
			email.setSslSmtpPort(Integer.toString(hostPort));
		}
		else
		{
			email.setSmtpPort(hostPort);
		}
		email.setTLS(tls);
		email.setCharset("UTF-8");
		email.setFrom(senderEmail, senderName, "UTF-8");
		email.addReplyTo(senderEmail, senderName, "UTF-8");
		return email;
	}
	
	
}
