/*
	$Id: Enumeration.java 5384 2010-08-13 08:50:27Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Johan Enell, Jari Häkkinen, Nicklas Nordborg
	Copyright (C) 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.LinkedList;

/**

	@author Nicklas
	@version 2.0
	@base.modified $Date: 2010-08-13 10:50:27 +0200 (Fri, 13 Aug 2010) $
*/
public class Enumeration<K, V>
	implements Serializable
{

	private static final long serialVersionUID = 1222952912319368574L;
	private final List<Entry<K, V>> entries;
	private boolean locked;

	public Enumeration()
	{
		entries = new LinkedList<Entry<K, V>>();
		locked = false;
	}
	
	public boolean isLocked()
	{
		return locked;
	}
	
	public void lock()
	{
		this.locked = true;
	}
	
	public int size()
	{
		return entries.size();
	}
	
	public int add(K key, V value)
	{
		if (locked) throw new UnsupportedOperationException("add; The enumeration has been locked for modifications.");
		int index = size();
		entries.add(new Entry<K, V>(key, value));
		return index;
	}
	
	public K getKey(int index)
	{
		return entries.get(index).getKey();
	}
	
	public V getValue(int index)
	{
		return entries.get(index).getValue();
	}
	
	public Entry<K, V> getEntry(int index)
	{
		return entries.get(index);
	}
	
	public void remove(int index)
	{
		if (locked) throw new UnsupportedOperationException("add; The enumeration has been locked for modifications.");
		entries.remove(index);
	}
	
	/**
		This method will sort the enumration on the keys using 
		{@link Collections#sort(List, Comparator)}. 
		The comparator will try to use the keys 'compareTo' method 
		if it implements the Comparable interface
		otherwilse it will treat the key as a String.
	*/
	public void sortKeys()
	{
		Collections.sort(entries, new Comparator<Entry<K, V>>()
		{
			@SuppressWarnings("unchecked")
			public int compare(Entry<K, V> o1, Entry<K, V> o2)
			{
				K key1 = o1.getKey();
				K key2 = o2.getKey();
				if (key1 instanceof Comparable)
				{
					return ((Comparable<K>) key1).compareTo(key2);
				}
				else
				{
					return key1.toString().compareTo(key2.toString());
				}
			}
		});
	}
	
	/**
		This method will sort the enumration on the values using 
		{@link Collections#sort(List, Comparator)}. 
		The comparator will try to use the values 'compareTo' method 
		if it implements the Comparable interface
		otherwilse it will treat the value as a String.
	*/	
	public void sortValues()
	{
		Collections.sort(entries, new Comparator<Entry<K, V>>()
		{
			@SuppressWarnings("unchecked")
			public int compare(Entry<K, V> o1, Entry<K, V> o2)
			{
				V value1 = o1.getValue();
				V value2 = o2.getValue();
				if (value1 instanceof Comparable)
				{
					return ((Comparable<V>) value1).compareTo(value2);
				}
				else
				{
					return value1.toString().compareTo(value2.toString());
				}
			}
		});
	}
	public static class Entry<K, V>
	{
		private final K key;
		private final V value;
		
		private Entry(K key, V value)
		{
			this.key = key;
			this.value = value;
		}
		
		public K getKey()
		{
			return key;
		}
		
		public V getValue()
		{
			return value;
		}
		
	}

	
	
}
