/**
	$Id: AnnotationWriter.java 5384 2010-08-13 08:50:27Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.bfs;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.HashSet;
import java.util.Set;

import net.sf.basedb.core.File;
import net.sf.basedb.util.encode.TabCrLfEncoderDecoder;
import net.sf.basedb.util.export.TableWriter;

/**

	@author Nicklas
	@version 2.15
	@base.modified $Date: 2010-08-13 10:50:27 +0200 (Fri, 13 Aug 2010) $
*/
public class AnnotationWriter
	extends TableWriter
{

	/**
		Utility method for creating an annotation writer when you have an
		output stream.
		@param out The output stream the annotation writer should print to
		@param filename Optional, the name of the file the output stream
			is printing to
	*/
	public static AnnotationWriter create(OutputStream out, String filename)
	{
		if (out == null) throw new NullPointerException("out");
		AnnotationWriter writer = new AnnotationWriter(
				new OutputStreamWriter(out, Charset.forName("UTF-8")));
		writer.setFilename(filename);
		return writer;
	}
	
	/**
		Utility method for creating an annotation writer to a file in the BASE
		file system. The character set and MIME type on the file are automatically 
		updated.
		@param file The file in the BASE file system
	*/
	public static AnnotationWriter create(File file)
	{
		if (file == null) throw new NullPointerException("file");
		file.setCharacterSet("UTF-8");
		file.setMimeType("text/plain");
		return create(file.getUploadStream(false), file.getName());
	}

	/**
		Utility method for creating an annotation writer to a file in the native
		file system. If the file doesn't exists, it is created.
		@param file The file in the native file system
	*/
	public static AnnotationWriter create(java.io.File file)
		throws IOException
	{
		if (file == null) throw new NullPointerException("file");
		if (!file.createNewFile() && !file.exists()) 
		{
			throw new FileNotFoundException(file.toString());
		}
		return create(new FileOutputStream(file), file.getName());
	}

	
	private String filename;
	private boolean hasPrintedHeaders;
	private int columnCount = -1;
	private Set<Integer> usedIds;
	
	public AnnotationWriter(Writer out)
	{
		super(out);
		super.setDataSeparator("\t");
		super.setNullValue("");
		super.setEncoder(new TabCrLfEncoderDecoder(false));
		this.usedIds = new HashSet<Integer>();
	}
	
	/**
		Get the file name that this writer is printing to.
		@return The file name or null if not known
	*/
	public String getFilename()
	{
		return filename;
	}
	
	/**
		Set the file name that this writer is printing to.
	*/
	public void setFilename(String filename)
	{
		this.filename = filename;
	}

	
	/**
		Checks if headers has already been printed or not.
	*/
	public boolean hasPrintedHeaders()
	{
		return hasPrintedHeaders;
	}
	
	/**
		Get the number of data columns. This information is only
		available after the headers has been printed.
		
		@return The number of column, or -1 if the headers has not yet been
			printed
	*/
	public int getColumnCount()
	{
		return columnCount;
	}
	
	/**
		Print the header line. This method must always be called before
		printing any data. It can only be called once. Each column header
		must be unique and 'ID' is not allowed since it is always used for
		the first column. Null values are not allowed. 
		
		@param headers The header columns, not including the ID column
		@throws IllegalStateException If headers has already been printed
		@throws NullPointerException If any of the header values is null
		@throws IllegalArgumentException If a duplicate header is found
	*/
	public void bfsPrintHeaders(String... headers)
	{
		if (hasPrintedHeaders())
		{
			throw new IllegalStateException("Headers has already been printed to file: " + 
					getFilename());
		}
		if (headers == null) 
		{
			throw new NullPointerException("'headers' in file: " + getFilename());
		}
		
		Set<String> used = new HashSet<String>();
		used.add("ID");
		for (int i = 0; i < headers.length; ++i)
		{
			String header = headers[i];
			if (header == null) 
			{
				throw new NullPointerException("'headers[" + i + "]' in file: " + getFilename());
			}
			if (used.contains(header))
			{
				throw new IllegalArgumentException("Duplicate header at index " + i + 
					" in file " + getFilename() + ": " + header);
			}
			used.add(header);
		}
		
		print("ID\t");
		tablePrintData((Object[])headers);
		hasPrintedHeaders = true;
		columnCount = headers.length;
	}
	
	/**
		Print a data line. Before this method can be used headers must have been
		printed. The size of the data array must be exactly the same as the
		size of the headers array.
		
		@param id The id of the line, must be a unique positive integer
		@param data The data to print
		@throws IllegalStateException If headers has not been printed
		@throws IllegalArgumentException If the ID values is not a unique postive
			integer or if the size of the data array doesn't match the size of the
			header array
		@throws NullPointerException If the data array is null
	*/
	public void bfsPrintData(int id, Object... data)
	{
		if (!hasPrintedHeaders())
		{
			throw new IllegalStateException("Must print headers before printing data to file: " + 
				getFilename());
		}
		if (id <= 0)
		{
			throw new IllegalArgumentException("The id must be a positive integer in file: " + 
				getFilename());
		}
		if (usedIds.contains(id))
		{
			throw new IllegalArgumentException("Id value must be unique in file " + 
				getFilename() + ": " + id);
		}
		if (data == null) throw new NullPointerException("'data' in file: " + getFilename());
		if (data.length != columnCount)
		{
			throw new IllegalArgumentException("Data array has " + data.length + " columns; " +
					"; expected " + columnCount + " in file: " + getFilename());
		}
		usedIds.add(id);
		print(id);
		if (data.length > 0)
		{
			print("\t");
			tablePrintData(data);
		}
		else
		{
			print("\n");
		}
	}
	
}
