/**
	$Id:Context.java 4187 2008-03-20 11:15:25Z nicklas $

	Copyright (C) Authors contributing to this file.

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.extensions;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.SessionControl;

/**
	Keeps information about the current context in the client 
	application that extensions can use to decide what actions to 
	generate. It is, for example, possible to check the permissions
	of logged in user and enable/disable certain actions
	if a condition is not met. Or, the extension may inspect
	the current item and decide if the extension can be used
	on that item or not.
	
	<p>
	The current context is passed to an {@link ActionFactory} twice, first
	to check if an extension should be enabled or not
	with the {@link ActionFactory#prepareContext(InvokationContext)}
	method. This method also allows an extension to write
	back information to the context. This may, for example,
	include information about which Javascript scripts to import
	or which stylesheets to load. The <code>prepareContext</code>
	method is only called once for each use of an extension.

	<p>
	The second time is when calling the {@link 
	ActionFactory#getActions(InvokationContext)} method. This method can
	be called multiple times. For example, if an extension point
	is in a list context, the <code>getActions</code> method may
	be called one time for each item in the list. If so, the
	{@link #getCurrentItem()} returns the currently active item.
	
	<p>
	Client applications may subclass this object to provide more
	information about the client environment. See, for example,
	the {@link net.sf.basedb.clients.web.extensions.JspContext} object.

	@author nicklas
	@version 2.7
	@base.modified $Date:2008-03-20 12:15:25 +0100 (Thu, 20 Mar 2008) $
*/
public class ClientContext
{

	private final SessionControl sc;
	private final DbControl dc;
	private Object item;
	private Map<String, Object> attributes;
	
	/**
		Create a new context with a session and no DbControl. The session 
		is final and can't be replaced.
		@param sc The current session
	*/
	public ClientContext(SessionControl sc)
	{
		this(sc, null, null);
	}
	
	/**
		Create a new context with a session and item but no DbControl. The session is 
		final and can't be replaced. Use {@link #setCurrentItem(Object)} to change 
		the item.
		
		@param sc The current session
		@param item The current item
	*/
	public ClientContext(SessionControl sc, Object item)
	{
		this(sc, null, item);
	}
	
	/**
		Create a new context with a DbControl. The DbControl 
		is final and can't be replaced.
		@param dc An open DbControl
	*/
	public ClientContext(DbControl dc)
	{
		this(dc.getSessionControl(), dc, null);
	}
	
	/**
		Create a new context with a DbControl and item. The DbControl is 
		final and can't be replaced. Use {@link #setCurrentItem(Object)} to change 
		the item.
		
		@param dc An open DbControl
		@param item The current item
	*/
	public ClientContext(DbControl dc, Object item)
	{
		this(dc.getSessionControl(), dc, item);
	}

	protected ClientContext(SessionControl sc, DbControl dc, Object item)
	{
		this.dc = dc;
		this.sc = sc;
		this.item = item;
	}
	
	/**
		Get the current session control.
	*/
	public SessionControl getSessionControl()
	{
		return sc;
	}

	/**
		Get the current DbControl. This can be null if the
		extension point doesn't provide an open DbControl.
		An extension MUST NEVER call {@link DbControl#close()}
		or {@link DbControl#commit()}.
	*/
	public DbControl getDbControl()
	{
		return dc;
	}
	
	/**
		Get the currently active item. It can be any type of
		object, but the extension point may declare (in it's 
		documentation) that the current item is always of a 
		certain class.
		@return The current item, or null
	*/
	public Object getCurrentItem()
	{
		return item;
	}
	
	/**
		Set the currently active item. 
		@param item The item that is the current item
	*/
	public void setCurrentItem(Object item)
	{
		this.item = item;
	}
	
	/**
		Get an attribute of the context.
		@param name The name of the attribute
		@return The value, or null
	*/
	public Object getAttribute(String name)
	{
		return attributes == null ? null : attributes.get(name);
	}
	
	/**
		Set an attribute of the context.
		@param name The name of the attribute
		@param value The value, if null the attribute is removed
	*/
	public void setAttribute(String name, Object value)
	{
		if (attributes == null) attributes = new HashMap<String, Object>();
		if (value != null)
		{
			attributes.put(name, value);
		}
		else
		{
			attributes.remove(name);
		}
	}
	
}
