/**
	$Id: ValidationOptions.java 5076 2009-08-25 09:15:41Z nicklas $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview;

import java.util.HashMap;
import java.util.Map;

import net.sf.basedb.core.Presets.Preset;

/**
	Stores all validation options (ie. validator -- severity) combinations.
	Note that all rules are always validated. Use object of this class to
	keep track of which validators that should be ignored, generate warnings or errors.

	@author Nicklas
	@version 2.2
	@base.modified $Date: 2009-08-25 11:15:41 +0200 (Tue, 25 Aug 2009) $
	@see GenericOverview#getValidationOptions()
	@see Failure
*/
public class ValidationOptions
{
	
	/**
		Maps Validator ID to severity.
	*/
	private Map<String, Severity> severities;
	
	/**
		Other options. Key is validator ID + key
	*/
	private Map<String, String> options;
	
	/**
		Create a new ValidationOptions object.
	*/
	public ValidationOptions()
	{
		this(null);
	}
	
	/**
		Create a new ValidationOptions object and load settings from a
		preset.
		@param preset The preset to load settings from, or null to not
			load any settings
	*/
	public ValidationOptions(Preset preset)
	{
		this.severities = new HashMap<String, Severity>();
		this.options = new HashMap<String, String>();
		if (preset != null) loadFromPreset(preset, true);
	}
	
	/**
		Get the severity level for the specified validator rule. If no option
		exists the validators default severity level is returned.
		@param validator The validator to get the severity level for
		@return A severity level object
		@see Validator#getDefaultSeverity()
	*/
	public Severity getSeverity(Validator validator)
	{
		Severity option = severities.get(validator.getId());
		if (option == null) option = validator.getDefaultSeverity();
		return option;
	}
	
	/**
		Set a severity level for the specified validation rule.
		@param validator The validation rule
		@param severity The severity level or null to use the default severity level
	*/
	public void setSeverity(Validator validator, Severity severity)
	{
		if (severity == null)
		{
			severities.remove(validator.getId());
		}
		else
		{
			severities.put(validator.getId(), severity);
		}
	}
	
	/**
		Get the configured option for a validator rule.
		
		@param validator The validator
		@param key The key (name) of the option
		@param defaultValue The default value in case there is no configured value
		@return The value of the option
		@since 2.13
	*/
	public String getOption(Validator validator, String key, String defaultValue)
	{
		String value =  options.get(validator.getId() + "+" + key);
		if (value == null) value = defaultValue;
		return value;
	}
	
	/**
		Set an option for a validator rule.
		
		@param validator The validator
		@param key The key (name) of the option
		@param value The value to set or null to remove the value
		@since 2.13
	*/
	public void setOption(Validator validator, String key, String value)
	{
		key = validator.getId() + "+" + key;
		if (value == null)
		{
			options.remove(key);
		}
		else
		{
			options.put(key, value);
		}
	}
	
	/**
		Load validation options from a {@link Preset}. The preset
		should contain settings where the ID of the a {@link Validator}
		is used as keys and the names of {@link Severity} values are used
		as values.
		@param preset The preset to load the options from
		@param clearFirst If the current settings should be cleared
			before the new ones are loaded from the preset
		@see #saveToPreset(net.sf.basedb.core.Presets.Preset)
	*/
	public void loadFromPreset(Preset preset, boolean clearFirst)
	{
		if (clearFirst) 
		{
			severities.clear();
			options.clear();
		}
		for (Map.Entry<String, String> entry : preset)
		{
			String key = entry.getKey();
			String value = entry.getValue();
			if (key.contains("+"))
			{
				options.put(key, value);
			}
			else
			{
				severities.put(key, Severity.valueOf(value));
			}
		}
	}
	
	/**
		Save the current settings to a preset. The {@link Validator#getId()}
		is used as key and the {@link Severity#name()} is used as value.
		@param preset The preset to save the settings to
	*/
	public void saveToPreset(net.sf.basedb.core.Presets.Preset preset)
	{
		for (Map.Entry<String, Severity> entry : severities.entrySet())
		{
			preset.setSetting(entry.getKey(), entry.getValue().name());
		}
		for (Map.Entry<String, String> entry : options.entrySet())
		{
			preset.setSetting(entry.getKey(), entry.getValue());
		}
	}
	
}
