/**
	$Id: ColFunction.java 4515 2008-09-11 20:08:14Z jari $

	Copyright (C) 2006, 2007 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.nfunk.jep.ParseException;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.util.jep.JepFunction;

/**
	Adds a <code>col()</code> function to Jep. The col function is used with the 
	{@link FlatFileParser} to get the value of a data column. The argument to
	the col function is either the column name or index (0-based). Use the 
	{@link FlatFileParser#getMapper(String)} with an argument starting with an equal
	sign to create a mapper that uses Jep.
	<pre class="code">
FlatFileParser ffp = ...
Mapper diameterMapper = ffp.getMapper("=col('Radius') * 2");
while (ffp.hasMoreData())
{
	Data data = ffp.nextData();
	float diameter = diameterMapper.getFloat(data);
}
</pre>

	@author nicklas
	@version 2.0
	@base.modified $Date: 2008-09-11 22:08:14 +0200 (Thu, 11 Sep 2008) $
*/
public class ColFunction
	implements JepFunction
{
	
	private final Map<String, Integer> columnHeaders;
	private final NumberFormat numberFormat;
	private final ParsePosition pos;
	private FlatFileParser.Data data;
	private boolean ignoreNonExistingColumns = false;
	
	public ColFunction(Map<String, Integer> columnHeaders, NumberFormat numberFormat)
	{
		this.columnHeaders = columnHeaders;
		this.numberFormat = numberFormat;
		this.pos = new ParsePosition(0);
	}
	
	public ColFunction(List<String> columnHeaders, NumberFormat numberFormat)
	{
		this.numberFormat = numberFormat;
		this.pos = new ParsePosition(0);
		this.columnHeaders = new LinkedHashMap<String, Integer>(columnHeaders.size());
		int index = 0;
		for (String header : columnHeaders)
		{
			this.columnHeaders.put(header, index);
			index++;
		}
	}
	
	/*
		From the JepFunction interface
		-------------------------------------------
	*/
	/**
		@return The string "col"
	*/
	public String getFunctionName()
	{
		return "col";
	}
	// -------------------------------------------
	/*
		From the PostfixMathCommandI interface
		-------------------------------------------
	*/
	/**
		@return Always 1
	*/
	public int getNumberOfParameters()
	{
		return 1;
	}
	public void setCurNumberOfParameters(int n)
	{}
	public boolean checkNumberOfParameters(int n)
	{
		return n == 1;
	}
	@SuppressWarnings({"unchecked"})
	public void run(Stack stack)
		throws ParseException
	{
		if (stack == null || stack.empty()) 
		{
			throw new ParseException("Stack is empty");
		}
		Object argument = stack.pop();
		String value = null;
		if (argument instanceof Number)
		{
			Number column = (Number)argument;
			value = data.get(column.intValue());
		}
		else if (argument instanceof String)
		{
			String colName = (String)argument;
			Integer column = columnHeaders.get(colName);
			if (column == null) column = findColumn(colName);
			if (column == null)
			{
				if (!ignoreNonExistingColumns)
				{
					throw new BaseException("Column '" + colName + "' not found in column headers.");
				}
				value = null;
			}
			else
			{
				value = data.get(column);
			}
		}
		else
		{
			throw new ParseException("Invalid parameter type: " + argument + "; expected number");
		}
		Double d = null;
		if (value != null)
		{
			if (numberFormat == null)
			{
				try
				{
					d = Double.valueOf(value);
				}
				catch (NumberFormatException ex)
				{
					stack.push(value);
					return;
				}
			}
			else
			{
				pos.setIndex(0);
				d = numberFormat.parse(value, pos).doubleValue();
				if (pos.getIndex() < value.length())
				{
					stack.push(value);
					return;
				}
			}
		}
		stack.push(d);
	}
	// -------------------------------------------

	/**
		Set to TRUE to ignore non-existing columns. Set to FALSE to
		throw an exception (default).
		@since 2.6
	*/
	public void setIgnoreNonExistingColumns(boolean ignoreNonExistingColumns)
	{
		this.ignoreNonExistingColumns = ignoreNonExistingColumns;
	}
	
	public void setData(FlatFileParser.Data data)
	{
		this.data = data;
	}
	
	/**
		Find column index by checking each column header against a
		regular expression. The first match found is returned. The
		map is updated with a new regex -> index entry.
		@return The column index or null if no column is found or if the 
			string is not a valid regular expression
	*/
	private Integer findColumn(String regex)
	{
		try
		{
			Pattern p = Pattern.compile(regex);
			Integer index = null;
			for (Map.Entry<String, Integer> entry : columnHeaders.entrySet())
			{
				String column = entry.getKey();
				if (p.matcher(column).matches())
				{
					index = entry.getValue();
					break;
				}
			}
			if (index != null) columnHeaders.put(regex, index);
			return index;
		}
		catch (PatternSyntaxException ex)
		{
			return null;
		}
	}

}
