/*
	$Id: ConstantMapper.java 5384 2010-08-13 08:50:27Z nicklas $
	
	Copyright (C) 2006 Nicklas Nordborg
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg
	
	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/
	
	BASE is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.
	
	BASE is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.parser;

import java.text.NumberFormat;

import net.sf.basedb.core.Type;
import net.sf.basedb.util.parser.FlatFileParser.Data;

/**
	A constant string mapper. The <code>getValue()</code> always
	return the same string which is set in the contructor. 

	@author nicklas
	@version 2.0
	@base.modified $Date: 2010-08-13 10:50:27 +0200 (Fri, 13 Aug 2010) $
*/
public class ConstantMapper
	implements Mapper
{
	private final String constant;
	private final NumberFormat parser;
	private final boolean nullIfException;
	private boolean needToParseNumeric;
	private Float asFloat;
	private Integer asInteger;
	private NumberFormatException parseException;
	
	/**
		Create a constant mapper.
	 	@param constant The constant expression. 
	*/
	public ConstantMapper(String constant)
	{
		this(constant, null);
	}
	
	/**
		Create a constant mapper using a specific number formatter as it's parser.
		@param constant The constant expression
		@param parser The number format to use or null to use Double.valueOf()
		@since 2.2
		@deprecated Use {@link ConstantMapper#ConstantMapper(String, NumberFormat, boolean)}
	*/
	public ConstantMapper(String constant, NumberFormat parser)
	{
		this(constant, parser, false);
	}
	
	/**
		Create a constant mapper using a specific number formatter as it's parser.
		@param constant The constant expression
		@param parser The number format to use or null to use Double.valueOf()
		@param nullIfException If TRUE, the mapper returns null for unparsable numeric
			values, otherwise an excption is thrown
		@since 2.4
	*/
	public ConstantMapper(String constant, NumberFormat parser, boolean nullIfException)
	{
		this.constant = constant;
		this.parser = parser;
		this.nullIfException = nullIfException;
		this.needToParseNumeric = constant != null;
	}
	
	public ConstantMapper(Integer constant)
	{
		this.constant = constant.toString();
		this.asInteger = constant;
		this.asFloat = constant.floatValue();
		this.parser = null;
		this.needToParseNumeric = false;
		this.nullIfException = false;
	}
	
	public ConstantMapper(Float constant)
	{
		this.constant = constant.toString();
		this.asFloat = constant;
		this.asInteger = constant.intValue();
		this.parser = null;
		this.needToParseNumeric = false;
		this.nullIfException = false;
	}
	
	/*
		From the Mapper interface
		-------------------------------------------
	*/
	/**
		@return The string set in the constructor
	*/
	public String getValue(Data data)
	{
		return constant;
	}
	public Integer getInt(Data data)
	{
		if (parseException != null) throw parseException;
		if (needToParseNumeric) parseNumeric();
		return asInteger;
	}
	public Float getFloat(Data data)
	{
		if (parseException != null) throw parseException;
		if (needToParseNumeric) parseNumeric();
		return asFloat;
	}
	// -------------------------------------------
	
	/*
		From the Object class
		-------------------------------------------
	*/
	public String toString()
	{
		return constant;
	}
	// -------------------------------------------

	private void parseNumeric()
	{
		assert constant != null : "constant == null";
		needToParseNumeric = false;
		try
		{
			if (parser != null)
			{
				asFloat = (Float)Type.FLOAT.parseString(constant, parser, nullIfException);
			}
			else
			{
				asFloat = Double.valueOf(constant).floatValue();
			}
		}
		catch (NumberFormatException ex)
		{
			if (!nullIfException) parseException = ex;
		}
		if (asFloat != null)
		{
			asInteger = asFloat.intValue();
		}
	}
}
