/*
$Id: PluginConfigurationImporter.java 5405 2010-09-10 11:09:05Z nicklas $

Copyright (C) 2006 Johan Enell, Jari Häkkinen, Martin Svensson
Copyright (C) 2007 Nicklas Nordborg

This file is part of BASE - BioArray Software Environment.
Available at http://base.thep.lu.se/

BASE is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

BASE is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/

package net.sf.basedb.plugins;

import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BooleanParameterType;
import net.sf.basedb.core.DateParameterType;
import net.sf.basedb.core.DateUtil;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.DoubleParameterType;
import net.sf.basedb.core.File;
import net.sf.basedb.core.FileParameterType;
import net.sf.basedb.core.FloatParameterType;
import net.sf.basedb.core.IntegerParameterType;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemContext;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.LongParameterType;
import net.sf.basedb.core.ParameterType;
import net.sf.basedb.core.PluginConfiguration;
import net.sf.basedb.core.PluginDefinition;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.AbstractPlugin;
import net.sf.basedb.core.plugin.AutoDetectingImporter;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.ImportInputStream;
import net.sf.basedb.core.plugin.InteractivePlugin;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.core.signal.SignalHandler;
import net.sf.basedb.core.signal.SignalTarget;
import net.sf.basedb.core.signal.ThreadSignalHandler;
import net.sf.basedb.util.XMLUtil;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

/**
This plugin imports pluginconfiguration from an XML-file.
The XML-file need to have a DOCLET definition in the first 30 lines
and the format in the XML-file should use: plugin-configuration-file.dtd in /net/sf/basedb/core/dtd/.
  
@author Martin
@version 2.1
@base.modified $Date$
*/

public class PluginConfigurationImporter
	extends AbstractPlugin
		implements InteractivePlugin, AutoDetectingImporter, SignalTarget
{	
	private static final String COMMAND_SELECT_CONFIGURATIONS = "select_configurations";
	
	private static final About about = new AboutImpl
	(
			"Plugin configuration importer",
			"This plugin is used to import configuration for a BASE plugin from an XML-file.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2006, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
	);
	
	private static final String dtdFileName = "plugin-configuration-file.dtd";
	
	private static final GuiContext pluginConfigurationList= new GuiContext(Item.PLUGINCONFIGURATION, GuiContext.Type.LIST);	
	private static final Set<GuiContext> guiContexts =  
		new HashSet<GuiContext>(Arrays.asList(pluginConfigurationList));
	
	// Job parameters
	private static final PluginParameter<File> xmlFileParameter = new PluginParameter<File> (
			"xmlfile",
			"XML file",
			"The XML-file to import the configuration parameters from",
			new FileParameterType(null, true, 1));
	private static final PluginParameter<Boolean> setPermissionsParameter = 
		new PluginParameter<Boolean>(
			"setPermissions",
			"Inherit shared permissions",
			"Select if the new configurations should be shared to the same users, " +
			"groups and projects as the plug-in they belong to. If FALSE, the new configurations " +
			"will not be shared.",
			new BooleanParameterType(true, true)
		);
	
	private static final PluginParameter<Boolean> importAllParameter = 
			new PluginParameter<Boolean>(
					"importAll",
					"Import all configurations",
					"Select if all or only a selection of the configurations in the XML-file should be " +
					"imported. If FALSE, the configurations " +
					"is selected in the next window.",
					new BooleanParameterType(true, true));
	
	private RequestInformation configureJob;
	private RequestInformation configureSelection;
	
	// Variables to store the job parameters in.
	private File xmlFile;
	private boolean setPermissions;
	private boolean importAll;
	private HashMap<Integer, Boolean> configurationMap;

	private ThreadSignalHandler signalHandler;
	
	/*
 		From the InteractivePlugin interface
	 	----------------------------------------------------
	*/	
	public Set<GuiContext> getGuiContexts()
	{
		return guiContexts;
	}

	public String isInContext(GuiContext context, Object item)
	{
		return null;
	}

	public RequestInformation getRequestInformation(GuiContext context, String command) 
		throws BaseException
		
	{
		RequestInformation requestinformation = null;
		if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{
			requestinformation = getConfiguredJobParameters();
			ItemContext fileContext = sc.getCurrentContext(Item.FILE);
			fileContext.removePropertyFilter("fileType"); // No existing filetype
		}	
		else if (command.equals(COMMAND_SELECT_CONFIGURATIONS))
		{
			requestinformation = getConfiguredSelectionParameters();
		}
		return requestinformation;
	}

	@SuppressWarnings("unchecked")
	public void configure(GuiContext context, Request request, Response response)
	{
		String command = request.getCommand();
		if (command.equals(Request.COMMAND_CONFIGURE_JOB))
		{	
			List<Throwable> errors = validateRequestParameters (
					getConfiguredJobParameters().getParameters(), 
					request
			); 
			
			if (errors != null)
			{
				response.setError(errors.size() + " invalid parameters were found in the request",errors);
				return;
			}
			storeValue(job, request, xmlFileParameter);
			storeValue(job, request, setPermissionsParameter);
			storeValue(job, request, importAllParameter);
			if (!(Boolean)request.getParameterValue("importAll"))
			{
				response.setContinue(COMMAND_SELECT_CONFIGURATIONS);
			}
			else
			{
				response.setDone("The job configuration is complete");
			}
		}
		else if (command.equals(COMMAND_SELECT_CONFIGURATIONS))
		{
			List<PluginParameter<?>> selectionParameters = getConfiguredSelectionParameters().getParameters(); 
			List<Throwable> errors = validateRequestParameters (
					selectionParameters,
					request
			);
			
			if (errors != null)
			{
				response.setError(errors.size() + "invalid parameters were found in the request", errors);
			}
			for (PluginParameter p : selectionParameters)
			{
				if (request.getParameterValue(p.getName()) != null)
				{
					storeValue(job, request, (PluginParameter<Boolean>)p);
				}
			}
			
			response.setDone("The job configuration is complete");
		}
	}

	/*
	 	From the Plugin interface
	 	-----------------------------------
	*/
	public MainType getMainType()
	{
		return MainType.IMPORT;
	}

	public About getAbout()
	{
		return about;
	}
	
	public boolean supportsConfigurations()
	{
		return false;
	}
	
	public boolean requiresConfiguration()
	{
		return false;
	}

	public void run(Request request, Response response, ProgressReporter progress)
	{
		if (signalHandler != null) signalHandler.setWorkerThread(null);
		String command = request.getCommand();
		if (command.equals(Request.COMMAND_EXECUTE))
		{
			InputStream is = null;			
			try
			{
				xmlFile = (File)job.getValue("xmlfile");
				setPermissions = Boolean.TRUE.equals(job.getValue("setPermissions")); 
				importAll = job.getValue("importAll") == null ? true : Boolean.TRUE.equals(job.getValue("importAll"));
				
				is = xmlFile.getDownloadStream(0);				
				boolean importable = isImportable(is);		
				is.close();
				if (!importable)
				{
					throw new InvalidDataException("File "+ xmlFile + " cannot be imported by this plugin");
				}
				is = xmlFile.getDownloadStream(0);
				doImport(is, progress);
				is.close();
				
				response.setDone("Configuration import finished succesfully!");
				
			}
			catch (Throwable ex)
			{
				if (is != null)					
				{
					try
					{
						is.close();
					}
					catch(IOException ioex)
					{}				
				}		
				response.setError(ex.getMessage(), Arrays.asList(ex));
			}	
		}
		else
		{
			response.setError("Unknown command: "+command, null);
		}
	}

	/*
	 	From the AutoDetectingImporter interface
	 	-------------------------------------------
	*/
	public boolean isImportable(InputStream in) 
		throws BaseException
	{
		boolean importable = false;		
		String charset = null;
		if (in instanceof ImportInputStream)
		{
			charset = ((ImportInputStream)in).getCharacterSet();
		}
		if (charset == null) charset = "UTF-8";
		BufferedReader reader = new BufferedReader(
				new InputStreamReader(in, Charset.forName(charset)));
		try
		{
			String headerLine = reader.readLine();			
			if (headerLine != null && headerLine.startsWith("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"))
			{
				//Look for the <!DOCTYPE definition in the first 30 lines 
				int numHeaderLines = 0;
				do
				{
					headerLine = reader.readLine();
					numHeaderLines++;
					if (headerLine != null &&  headerLine.contains("<!DOCTYPE") )
					{
						StringBuilder docTypeTag = new StringBuilder(headerLine);
						while(headerLine != null && !headerLine.contains(">") && !(numHeaderLines > 30) )
						{
							headerLine = reader.readLine();
							numHeaderLines++;
							docTypeTag.append(headerLine);
						}						
						importable = docTypeTag.toString().contains(dtdFileName);
					}					
				}
				while( !importable && !(numHeaderLines > 30 ) );
			}
		}
		catch (IOException ioex)
		{
			throw new BaseException(ioex);
		}		
		return importable;
	}

	public void doImport(InputStream in, ProgressReporter progress) 
		throws BaseException
	{						
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();			
			String filePath = null;
			if (xmlFile != null)
			{
				File f = File.getById(dc, xmlFile.getId());
				filePath = f.getPath().toString();
			}
			//Get the root element of the XML file
			URL dtdURL = PluginConfigurationImporter.class.getResource("/net/sf/basedb/core/dtd/"+dtdFileName);
			Document doc = XMLUtil.getValidatedXML(in, dtdURL, filePath);		
			Element rootElement = doc.getRootElement();		
			
			int parameternumber = 0;
			List configurations = rootElement.getChildren();
			for (Object obj : configurations)
			{
				ThreadSignalHandler.checkInterrupted();
				Element configuration = (Element)obj;
				String name = configuration.getChildText("configname");
				String description = configuration.getChildText("description");
				String pluginClassName = configuration.getAttributeValue("pluginClassName");
				PluginDefinition pluginDefinition = null;
				try 
				{
					pluginDefinition = PluginDefinition.getByClassName(dc, pluginClassName);
					
					boolean importConfiguration = !importAll && getConfigurationInstallOption(parameternumber);
					if ((importConfiguration || importAll))
					{
						PluginConfiguration pluginConfig = PluginConfiguration.getNew(dc, pluginDefinition);
						if (setPermissions)
						{
							pluginConfig.setItemKey(pluginDefinition.getItemKey());
							pluginConfig.setProjectKey(pluginDefinition.getProjectKey());
						}
						pluginConfig.setName(name);
						pluginConfig.setDescription(description);
						dc.saveItem(pluginConfig);
						
						List parameters = configuration.getChildren("parameter");
						setPluginConf(parameters, pluginConfig);
					}
				}
				catch (ItemNotFoundException infex)
				{}
				
				parameternumber++;
			}				
			dc.commit();
		}
		catch (Throwable ex)
		{
			throw new BaseException(ex);			
		}
		finally
		{
			if (dc != null)
			{
				dc.close();
			}
		}
	}
	
	/*
	  	-----------------------------------------------------
	*/
	/*
		From the SignalTarget interface
		-------------------------------------------
	*/
	public SignalHandler getSignalHandler()
	{
		signalHandler = new ThreadSignalHandler();
		return signalHandler;
	}
	// -------------------------------------------
	
	/**
		Imports plugin configurations direct from an xml file, located in a jar.
		@since 2.5
		@param jarPath Path to the jar-file
		@param xmlPath Where in the jar-file the XML-file is located
		@param configurations A hasmap with a boolean value for each configuration in the xml-file. 
			The value should be TRUE if the configuration should be imported, FALSE otherwise.
		@param importAll If all configurations in the xml-file should be imported or not.
	  	@param setPermissions If the configurations should be shared to the same users as the plugins they belong to. 
	  	@throws IOException When something goes wrong accessing a file
	  	@throws JDOMException When failed to read the XML-file
	  	@throws BaseException When the hashmap with import options has different 
	  		size compared to number of configurations in XML-file
		
	 */
	public void importPluginConfigurationsFromJar(String jarPath, String xmlPath, 
				HashMap<Integer, Boolean> configurations,  boolean importAll, boolean setPermissions) 
		throws IOException, JDOMException, BaseException
	{
		this.importAll = importAll;
		this.setPermissions = setPermissions;
		this.configurationMap = configurations;
		
		JarFile jar = new JarFile(jarPath);
		InputStream is = jar.getInputStream(new JarEntry(xmlPath));
		doImport(is, null);
		is.close();
	}
	
	
	private RequestInformation getConfiguredJobParameters()
	{
		if (configureJob == null)
		{	
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
						
			parameters.add(xmlFileParameter);
			parameters.add(setPermissionsParameter);
			parameters.add(importAllParameter);
			
			configureJob = new RequestInformation
			(
				Request.COMMAND_CONFIGURE_JOB,
				"Configuration import options",
				"Set the options to run this job.\n" +
				"NOTICE! Only configurations for already installed plugins will be handled.",
				parameters
			);
		}
		return configureJob;
	}
	
	/**
	 	Gets request information to set which configuration to import
	  	@return request information with parameter to use.
	*/
	private RequestInformation getConfiguredSelectionParameters()
	{
		if (configureSelection == null)
		{
			Map<String, List<PluginParameter<?>>> configLists = new HashMap<String, List<PluginParameter<?>>>();
			
			DbControl dc = null;
			
			try
			{
				dc = sc.newDbControl();
				//Get the root element of the XML file
				String filePath = null;
				File file = (File)job.getValue("xmlfile");
				{
					file = File.getById(dc, file.getId());
					filePath = file.getPath().toString();
				}
				URL dtdURL = PluginConfigurationImporter.class.getResource("/net/sf/basedb/core/dtd/"+dtdFileName);
				InputStream is = file.getDownloadStream(0);
				Document doc = XMLUtil.getValidatedXML(is, dtdURL, filePath);
				is.close();
				Element rootElement = doc.getRootElement();
				
				List<PluginParameter<?>> typeList = null;
				List configurations = rootElement.getChildren();
				int parameternumber = 0;
				for (Object obj : configurations)
				{
					Element configElement = (Element)obj;
					String configName = configElement.getChildText("configname");
					String description = configElement.getChildText("description");
					String pluginName = configElement.getAttributeValue("pluginClassName");
					try
					{
						PluginDefinition.getByClassName(dc, pluginName);
						if (configLists.get(pluginName) == null)
						{
							typeList = new ArrayList<PluginParameter<?>>();
							typeList.add(new PluginParameter<Boolean>
								(
										pluginName, 
										pluginName.substring(pluginName.lastIndexOf('.') + 1), 
										"Configurations for plugin: " +pluginName, 
										null
								));
						}
						else
						{
							typeList = configLists.get(pluginName);
						}
						
						PluginParameter<Boolean> configurationParameter = 
							new PluginParameter<Boolean>(
							"configuration"+parameternumber,
							configName,
							"Select if configuration should be imported or not. " +
							"TRUE will include this configuration in the import.\n\n" + description,
							new BooleanParameterType(null, false)
						);
						typeList.add(configurationParameter);
						configLists.put(pluginName, typeList);
					}
					catch (ItemNotFoundException infex){}
					parameternumber++;
				}
				List<PluginParameter<?>>parameters = new ArrayList<PluginParameter<?>>();
				for (List<PluginParameter<?>> list : configLists.values())
				{
					parameters.addAll(list);
				}
				configureSelection = new RequestInformation
				(
						COMMAND_SELECT_CONFIGURATIONS,
						"Select configurations",
						"Select which plugin-configurations to import",
						parameters
				);
			}
			catch (Throwable ex)
			{
				throw new BaseException(ex);			
			}
			finally
			{
				if (dc != null)
				{
					dc.close();
				}
			}
		}
		return configureSelection;
	}
	
	/**
	  	Gets the list of values for a parameter from the XML-file.
	  	Returns a list with parameter values.
	*/ 	
	private List getValueList(Element parameter, Class classType)
	{	
		List children = parameter.getChildren("value");
		List<Object> values = new ArrayList<Object>();
		for (Object ch : children)
		{
			String text = ((Element)ch).getText();
			if (classType.equals(Boolean.class)) values.add(Boolean.valueOf(text));
			else if (classType.equals(Date.class)) values.add(DateUtil.parseString(text));
			else if (classType.equals(Double.class)) values.add(Double.parseDouble(text));
			else if (classType.equals(Float.class)) values.add(Float.parseFloat(text));
			else if (classType.equals(Long.class)) values.add(Long.parseLong(text));
			else if (classType.equals(String.class)) values.add(text);
		}		
		return values;
	}
	
	/**
	   	Gets the ParameterType to use with a class.
	   	Returns null if this importer doesn't support the class.
	*/ 
	private ParameterType getParameterType(Class parameterClass, List<?> values)
	{
		int multiplicity = 1;
		if (values != null && values.size() > 1) multiplicity = values.size();
		ParameterType pType = null;
		
		if (parameterClass.equals(Boolean.class))
		{
			pType = new BooleanParameterType(null, false, multiplicity, null);
		}
		else if (parameterClass.equals(Date.class)) 
		{
			pType = new DateParameterType(null, false, multiplicity, 0, 0, null);
		}
		else if (parameterClass.equals(Double.class)) 
		{
			pType = new DoubleParameterType(null, null, null, false, multiplicity, 0, 0, null);		
		}
		else if (parameterClass.equals(Float.class)) 
		{
			pType = new FloatParameterType(null, null, null, false, multiplicity, 0, 0, null);
		}
		else if (parameterClass.equals(Integer.class))
		{
			pType = new IntegerParameterType(null, null, null, false, multiplicity, 0, 0, null);		
		}
		else if (parameterClass.equals(Long.class)) 
		{
			pType = new LongParameterType(null, null, null, false, multiplicity, 0, 0, null);
		}
		else if (parameterClass.equals(String.class)) 
		{
			// Find longest string
			int maxLength = 255;
			if (values != null && !values.isEmpty())
			{
				for (Object obj : values)
				{
					if (obj instanceof String)
					{
						String str = obj.toString();
						if (str.length() > maxLength) maxLength = str.length();
					}
				}
			}
			pType = new StringParameterType(maxLength, null, false, multiplicity, 0, 0);
		}
		return pType;
	}
	
	/**
	 	Sets parameters for a PluginConfiguration
	  	@param parameters A list of the parameters that shall be set to the configuration
	  	@param plc The @link{PluginConfiguration} that the parameters shall be added to.
	  	@throws ClassNotFoundException If there is an error while trying to find the type of the parameters.
	*/
	private void setPluginConf(List parameters, PluginConfiguration plc)
		throws ClassNotFoundException
	{
		// Set each configuration parameter for the plugin.
		for (Object elementObj : parameters)
		{	
			Element parameter = (Element)elementObj;
			String name = parameter.getChildText("name");								
			String cl = parameter.getChildText("class"); 
			String label = parameter.getChildText("label");
			if (label == null || label.length() == 0) label = name; 	
			String description = parameter.getChildText("description");
			Class clazz = null;
			List<?> values = null;
			
			if (cl.length() > 0)
			{	
				clazz = Class.forName(cl);
				values = getValueList(parameter, clazz);
				ParameterType pType = getParameterType(clazz, values);
				plc.setParameterValues(name,label, description, pType, values);
			}
		}
	}
	
	/**
	 	Returns if a configuration in a certain place/order in the XML-file should be imported
	  	@param orderNumber The number of order the configuration has in the XML-file
	  	@return TRUE if the configuration should be imported, FALSE otherwise.
	 */
	private boolean getConfigurationInstallOption(int orderNumber)
	{
		boolean importIt = false;
		if (configurationMap != null)
		{
			Boolean value = configurationMap.get(orderNumber);
			importIt = value != null ? value : false;
		}
		else if (job != null)
		{
			importIt = job.getValue("configuration"+orderNumber) != null ? 
					Boolean.TRUE.equals(job.getValue("configuration" + orderNumber)) : false;
		}
		
		return importIt;
	}
}
