/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.executor;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.AnnotationType;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BioAssay;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.BooleanParameterType;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Formula;
import net.sf.basedb.core.InvalidDataException;
import net.sf.basedb.core.Nameable;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.StringParameterType;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.plugins.BaseFileExporterPlugin;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.basefile.BaseFileWriter;
import net.sf.basedb.util.export.spotdata.AdvancedFieldConverter;
import net.sf.basedb.util.export.spotdata.AnnotationAssayField;
import net.sf.basedb.util.export.spotdata.AssayField;
import net.sf.basedb.util.export.spotdata.BaseFileExporter;
import net.sf.basedb.util.export.spotdata.DynamicField;
import net.sf.basedb.util.export.spotdata.ExportableFieldConverter;
import net.sf.basedb.util.export.spotdata.ExportableFieldFactory;
import net.sf.basedb.util.export.spotdata.MatrixBaseFileExporter;
import net.sf.basedb.util.export.spotdata.SerialBaseFileExporter;
import net.sf.basedb.util.formatter.IdentifiableFormatter;
import net.sf.basedb.util.formatter.MultiFormatter;
import net.sf.basedb.util.formatter.NameableFormatter;
import net.sf.basedb.util.formatter.ToStringFormatter;

/**
	Exporter implementation for the external program executor that
	exports data to BASEfile format.
	
	@author nicklas
	@since 2.16
	@base.modified $Date $
*/
public class BaseFileExporterSupport
	extends AbstractIOSupport
{

	private static final About about =
		new AboutImpl
		(
			"BASEFile exporter for the ExternalProgramExecutor",
			"Support plug-in for the external program executor that exports data to the " +
			"external program in BASEFile format. This plug-in can't be used as a standalone " +
			"plug-in.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2010, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
		);

	
	public BaseFileExporterSupport()
	{}

	/*
		From the Plugin interface
		-------------------------
	*/
	@Override
	public About getAbout()
	{
		return about;
	}
	@Override
	public MainType getMainType()
	{
		return Plugin.MainType.EXPORT;
	}	
	// -------------------------

	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	@Override
	public void configure(GuiContext context, Request request, Response response)
	{
		try
		{
			String command = request.getCommand();
			if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_PLUGIN))
			{
				RequestInformation ri = getConfigureExecutorParameters(command);
				List<Throwable> errors = validateRequestParameters(ri.getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found " +
														"in the request", errors);
					return;
				}
				
				// Check that fields are unique
				try
				{
					checkUniqueFields((String)request.getParameterValue("spotFields"), "spot fields");
					checkUniqueFields((String)request.getParameterValue("reporterFields"), "reporter fields");
					//checkUniqueFields((String)request.getParameterValue("assayFields"), "assay fields");
				}
				catch (InvalidDataException ex)
				{
					response.setError(ex.getMessage(), null);
					return;
				}
				
				// Export options
				storeValue(configuration, request, ri.getParameter(BaseFileExporterPlugin.PARAMETER_FILEFORMAT));
				storeValue(configuration, request, ri.getParameter("datafile"));
				storeValue(configuration, request, ri.getParameter("averageOnReporter"));
				storeValue(configuration, request, ri.getParameter("spotFields"));
				storeValue(configuration, request, ri.getParameter("reporterFields"));
				storeValue(configuration, request, ri.getParameter("assayFields"));
				response.setDone("Executor configuration complete");
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
	}

	@Override
	public RequestInformation getRequestInformation(GuiContext context, String command) 
		throws BaseException
	{
		RequestInformation requestInformation = null;
		if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_PLUGIN))
		{
			requestInformation = getConfigureExecutorParameters(command);
		}
		return requestInformation;
	}
	// -----------------------------------------
	/*
		From the IOSupport interface
		----------------------------
	*/
	@Override
	public String getShortName()
	{
		return "BASEfile exporter";
	}
	// ----------------------------

	private RequestInformation configureExecutor;
	
	private RequestInformation getConfigureExecutorParameters(String command)
	{
		if (configureExecutor != null) return configureExecutor;
	
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();
			
			// File format parameter
			parameters.add(BaseFileExporterPlugin.getFileFormatParameter());
			// Name of the output file from BASE = input file to plug-in
			parameters.add(new PluginParameter<String>(
				"datafile", 
				"Data filename", 
				"The name of the file that the external program should read data from. The defult value " +
				"is 'stdin.txt' which is is automatically sent to the program on standard input. It " +
				"is possible to use other filenames but the program is then responsible to open the " +
				"file itself.",
				new StringParameterType(255, "stdin.txt", true))
			);

			// Average on reporter
			parameters.add(new PluginParameter<Boolean>("averageOnReporter",
					"Average on reporters",
					"Average on reporters within each assay. If false or not specified " +
					"no averaging is done",
					new BooleanParameterType(null, false)
				));
			
			// Exported fields
			parameters.add(new PluginParameter<String>(
					"reporterFields",
					"Reporter fields",
					"Name all reporter fields that you wish to include in the export. " +
					"Use one value per line. The exporter will try to find matching " +
					"fields as follows:\n" +
					" 1. Extra value\n" +
					" 2. Formula\n" +
					" 3. Static properties: Name, ExternalId, etc.\n" +
					"It is possible to force a specific case:\n" +
					" 1. =xtra(\"name\", [\"title\"])\n" +
					" 2. =formula(\"name\", [\"title\"])\n" +
					" 3. =property(\"name\", [\"title\"])\n" +
					" 4. =jep(\"expression\", \"title\", [\"datatype\"])\n" +
					"The last case is like 'formula' but doesn't require it to be registered in " +
					"the database before use.\n" + 
					"NOTE! Position and Internal id are required to be able to import\n" +
					"data again.",
					new StringParameterType(65535, "Position\nInternal id", false)
				));

			parameters.add(new PluginParameter<String>(
					"spotFields",
					"Spot fields",
					"Name all spot fields that you wish to include in the export. " +
					"Use one value per line. The exporter will try to find matching " +
					"fields as follows:\n" +
					" 1. Extra value\n" +
					" 2. Formula\n" +
					" 3. Raw data property\n" +
					" 4. Channel intensity: Ch 1, Ch 2, etc.\n" +
					"It is possible to force a specific case:\n" +
					" 1. =xtra(\"name\", [\"title\"])\n" +
					" 2. =formula(\"name\", [\"title\"])\n" +
					" 3. =property(\"name\", [\"title\"])\n" +
					" 4. =jep(\"expression\", \"title\", [\"datatype\"])\n" +
					"The last case is like 'formula' but doesn't require it to be registered in " +
					"the database before use.",
					new StringParameterType(65535, null, false)
				));

			parameters.add(new PluginParameter<String>(
					"assayFields",
					"Assay fields",
					"Name all assay fields that you wish to include in the export. " +
					"Use one value per line. The exporter will try to find matching " +
					"fields as follows:\n" +
					" 1. Experimental factor\n" +
					" 2. Assay annotation\n" +
					" 3. Static properties: Name, Description or Id\n" +
					"It is possible to force a specific case:\n" +
					" 1. =factor(\"name\", [\"title\"])\n" +
					" 2. =annotation(\"name\", [\"title\"])\n" +
					" 3. =property(\"name\", [\"title\"])\n" +
					"NOTE! If no fields are selected, the Id, Name and all experimental " +
					"factor values are automatically exported.",
					new StringParameterType(65535, null, false)
				));

			configureExecutor = new RequestInformation(
					command,
					"Select export options.",
					"Select file format options and which fields to export. Additional " +
					"fields may be defined as job parameters and selected at runtime.",
					parameters
				);
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return configureExecutor;
	}

	private void checkUniqueFields(String fields, String parameter)
	{
		if (fields == null) return;
		Set<String> all = new HashSet<String>();
		for (String fld : fields.split("\\n"))
		{
			if (!all.add(fld.trim())) 
			{
				throw new InvalidDataException("Duplicate entry in " + parameter + ": " + fld);
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	protected String performIO(DbControl dc, ProgressReporter progress)
		throws IOException
	{
		// Get job configuration parameters
		ExternalProgramExecutor executor = getExecutor();
		BioAssaySet source = executor.getCurrentBioAssaySet(dc);
		java.io.File workDir = executor.getWorkingDirectory();
		String format = (String)configuration.getValue(BaseFileExporterPlugin.PARAMETER_FILEFORMAT);
		String datafile = Values.getString((String)configuration.getValue("datafile"), "stdin.txt");
		boolean averageOnReporter = Boolean.TRUE.equals(configuration.getValue("averageOnReporter"));
		
		// Reporter fields
		ExportableFieldConverter fieldConverter = new AdvancedFieldConverter(dc, source);
		List<DynamicField> reporterFields = new ArrayList<DynamicField>();
		String tmp = (String)configuration.getValue("reporterFields");
		if (tmp != null)
		{
			for (String reporterField : tmp.split("\\n"))
			{
				reporterFields.add(fieldConverter.getReporterField(reporterField.trim(), true));
			}
		}
		// Spot fields
		List<DynamicField> spotFields = new ArrayList<DynamicField>();
		tmp = (String)configuration.getValue("spotFields");
		if (tmp != null)
		{
			for (String spotField : tmp.split("\\n"))
			{
				spotFields.add(fieldConverter.getSpotField(spotField.trim(), true));
			}
		}
		// Assay fields
		List<AssayField> assayFields = new ArrayList<AssayField>();
		tmp = (String)configuration.getValue("assayFields");
		if (tmp != null)
		{
			for (String assayField : tmp.split("\\n"))
			{
				assayFields.add(fieldConverter.getAssayField(assayField.trim(), true));
			}
		}

		List<ProgramParameter> programParameters = executor.getProgramParameters();
		
		// Create the BASEFile exporter
		if (progress != null) progress.display(0, "Exporting ...");
		BaseFileExporter exporter = null;
		if (SerialBaseFileExporter.SUBTYPE.equals(format))
		{
			exporter = new SerialBaseFileExporter();
		}
		else if (MatrixBaseFileExporter.SUBTYPE.equals(format))
		{
			exporter = new MatrixBaseFileExporter();
		}
		else
		{
			throw new BaseException("Unknown format. "+format);
		}
		exporter.setDbControl(dc);
		exporter.setProgressReporter(progress);
		exporter.setSource(source);
		exporter.setAverageOnReporter(averageOnReporter);
		
		// Create files & file factory
		java.io.File out = executor.createFile(datafile);
		exporter.setWriter(new BaseFileWriter(new FileWriter(out)));

		// Export parameter values
		MultiFormatter parameterFormatter = new MultiFormatter(new ToStringFormatter<Object>(), true);
		parameterFormatter.registerFormatter(Nameable.class, new NameableFormatter());
		parameterFormatter.registerFormatter(BioAssay.class, new IdentifiableFormatter());
		for (ProgramParameter pp : programParameters)
		{
			ParameterDefinition pdef = pp.getParameterDefinition();
			String name = pdef.getName();
			List values = pp.getValues();
			String parameterType = pdef.getType();
			if (values != null && values.size() > 0)
			{
				exporter.setParameter(name, Values.getString(values, ",", true, parameterFormatter));
				if ("factor".equals(parameterType))
				{
					// Add annotation fields to the exporter
					for (Object annotationType : values)
					{
						AnnotationAssayField fld = ExportableFieldFactory.annotation((AnnotationType)annotationType, null, null);
						if (!assayFields.contains(fld)) assayFields.add(fld);
					}
				}
				else if ("spotformula".equals(parameterType))
				{
					// Add formula fields to the exporter
					for (Object formula : values)
					{
						DynamicField fld = ExportableFieldFactory.formula((Formula)formula, null, null);
						if (!spotFields.contains(fld)) spotFields.add(fld);
					}
				}
			}
		}
		
		// Reporter fields
		if (reporterFields.size() > 0)
		{
			exporter.addReporterFields(reporterFields);
		}
		
		// Spot fields
		if (spotFields.size() > 0)
		{
			exporter.addSpotFields(spotFields);
		}

		// Assay fields
		if (assayFields.size() > 0)
		{
			exporter.addAssayFields(assayFields);
		}

		exporter.doExport();
		return "Bioassay set " + source.getName() + " successfully exported to " + workDir;
	}
	
}
