/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.plugins.executor;

import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.BioAssaySet;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Directory;
import net.sf.basedb.core.Experiment;
import net.sf.basedb.core.File;
import net.sf.basedb.core.ItemNotFoundException;
import net.sf.basedb.core.Job;
import net.sf.basedb.core.Path;
import net.sf.basedb.core.PathParameterType;
import net.sf.basedb.core.PluginParameter;
import net.sf.basedb.core.ProgressReporter;
import net.sf.basedb.core.RequestInformation;
import net.sf.basedb.core.Transformation;
import net.sf.basedb.core.User;
import net.sf.basedb.core.Version;
import net.sf.basedb.core.plugin.About;
import net.sf.basedb.core.plugin.AboutImpl;
import net.sf.basedb.core.plugin.GuiContext;
import net.sf.basedb.core.plugin.Plugin;
import net.sf.basedb.core.plugin.Request;
import net.sf.basedb.core.plugin.Response;
import net.sf.basedb.plugins.util.Parameters;
import net.sf.basedb.util.FileUtil;

/**
	Importer implementation for the external program executor that
	uploads all created files to BASE and attaches them to a child
	bioassay set.
	
	@author nicklas
	@since 2.16
	@base.modified $Date $
*/
public class FileOnlyImporterSupport
	extends AbstractIOSupport
{

	private static final About about =
		new AboutImpl
		(
			"File-only importer for the ExternalProgramExecutor",
			"Support plug-in for the external program executor that uploads all created files " +
			"to BASE and attaches them to a child bioassay set. This plug-in can't be " +
			"used as a standalone plug-in.",
			Version.getMajor() + "." + Version.getMinor() + "." + Version.getMaintenance(),
			"2010, Base 2 development team",
			null,
			null,
			"http://base.thep.lu.se"
		);

	
	public FileOnlyImporterSupport()
	{}

	/*
		From the Plugin interface
		-------------------------
	*/
	@Override
	public About getAbout()
	{
		return about;
	}
	@Override
	public MainType getMainType()
	{
		return Plugin.MainType.IMPORT;
	}
	// -------------------------

	/*
		From the InteractivePlugin interface
		-------------------------------------------
	*/
	@Override
	public void configure(GuiContext context, Request request, Response response)
	{
		try
		{
			String command = request.getCommand();
			if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_PLUGIN))
			{
				/*
				RequestInformation ri = getConfigureExecutorParameters(command);
				List<Throwable> errors = validateRequestParameters(ri.getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found " +
														"in the request", errors);
					return;
				}
				*/
				
				// Import options
				//storeValue(configuration, request, ri.getParameter("datafile"));
				//storeValue(configuration, request, ri.getParameter("extraFiles"));

				response.setDone("File-only importer configuration complete");
			}
			else if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_JOB))
			{
				RequestInformation ri = getConfigureJobParameters(command);
				List<Throwable> errors = validateRequestParameters(ri.getParameters(), request);
				if (errors != null)
				{
					response.setError(errors.size()+" invalid parameter(s) were found " +
														"in the request", errors);
					return;
				}
			
				storeValue(job, request, ri.getParameter(Parameters.CHILD_NAME));
				storeValue(job, request, ri.getParameter(Parameters.CHILD_DESCRIPTION));
				storeValue(job, request, ri.getParameter(Parameters.TRANSFORMATION_NAME));
				storeValue(job, request, ri.getParameter("extraFilesDirectory"));
				response.setDone("File-only importer configuration complete");
			}
		}
		catch (Throwable ex)
		{
			response.setError(ex.getMessage(), Arrays.asList(ex));
		}
	}

	@Override
	public RequestInformation getRequestInformation(GuiContext context, String command) 
		throws BaseException
	{
		RequestInformation requestInformation = null;
		if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_PLUGIN))
		{
			requestInformation = getConfigureExecutorParameters(command);
		}
		else if (command.equals(IOSupport.COMMAND_CONFIGURE_IO_JOB))
		{
			requestInformation = getConfigureJobParameters(command);
		}
		return requestInformation;
	}
	// -----------------------------------------
	/*
		From the IOSupport interface
		----------------------------
	*/
	@Override
	public String getShortName()
	{
		return "File-only importer";
	}
	// ----------------------------

	@Override
	protected String performIO(DbControl dc, ProgressReporter progress)
		throws IOException
	{
		// Get job configuration parameters
		ExternalProgramExecutor executor = getExecutor();
		BioAssaySet source = executor.getCurrentBioAssaySet(dc);
		String transformationName = (String)job.getValue(Parameters.TRANSFORMATION_NAME);
		String childName = (String)job.getValue(Parameters.CHILD_NAME);
		String childDescription = (String)job.getValue(Parameters.CHILD_DESCRIPTION);
		String extraFileDirectory = (String)job.getValue("extraFilesDirectory");
		
		// Create child transformation and bioassay set
		Job currentJob = getCurrentJob(dc);
		Transformation t = source.newTransformation(currentJob);
		t.setName(transformationName);
		dc.saveItem(t);
		BioAssaySet child = t.newProduct(null, null, true);
		child.setName(childName);
		child.setDescription(childDescription);
		dc.saveItem(child);

		// Import files
		if (extraFileDirectory != null)
		{
			final List<java.io.File> extraFiles = executor.inspectWorkingDirectory();
			if (extraFiles.size() > 0)
			{
				Directory extraDir = getExtraFilesDirectory(dc, extraFileDirectory);
				FileFilter newFiles = new FileFilter()
				{
					@Override
					public boolean accept(java.io.File pathname)
					{
						return pathname.isDirectory() || extraFiles.contains(pathname);
					}
				};
				List<File> uploaded = FileUtil.uploadFiles(dc, extraDir, executor.getWorkingDirectory(), 
						newFiles, false, progress);
				BasicItem from = child == null ? t : child;
				AnyToAny ata = AnyToAny.getNew(dc, from, extraDir, "./", false);
				dc.saveItem(ata);
				for (File f : uploaded)
				{
					Directory d = f.getDirectory();
					if (d == extraDir)
					{
						// Attach file to child bioassay set or transformation
						ata = AnyToAny.getNew(dc, from, f, "./" + f.getName(), false);
						dc.saveItem(ata);
					}
					else if (d.getParent() == extraDir)
					{
						// Attach directory to child bioassay set or transformation
						ata = AnyToAny.getNew(dc, from, d, "./" + d.getName(), false);
						dc.saveItem(ata);
					}
				}
			}
		}
		String msg = "Bioassay set '" + child.getName() + "' successfully created";
		return msg;
	}

	// No configuration-time parameters are needed
	private RequestInformation getConfigureExecutorParameters(String command)
	{
		return null;
	}

	private RequestInformation configureJob;
	private RequestInformation getConfigureJobParameters(String command)
	{
		if (configureJob != null) return configureJob;
	
		DbControl dc = null;
		try
		{
			dc = sc.newDbControl();
			List<PluginParameter<?>> parameters = new ArrayList<PluginParameter<?>>();

			String transformationName = configuration.getPluginConfiguration().getName();
			parameters.add(Parameters.transformationNameParameter(null, null, transformationName));
			parameters.add(Parameters.childNameParameter(null, null, "After " + transformationName));
			parameters.add(Parameters.childDescriptionParameter(null, null, null));
			
			Experiment experiment = getExecutor().getCurrentBioAssaySet(dc).getExperiment();
			String homeDirectoryPath = "/" + Path.makeSafeFilename(transformationName, "");
			Directory homeDirectory = experiment == null ? null : experiment.getDirectory();
			if (homeDirectory != null)
			{
				homeDirectoryPath = homeDirectory.getPath() + homeDirectoryPath;
			}
			else
			{
				homeDirectory = User.getById(dc, sc.getLoggedInUserId()).getHomeDirectory();
				if (homeDirectory != null)
				{
					homeDirectoryPath = homeDirectory.getPath() + homeDirectoryPath;
				}
			}
			parameters.add(new PluginParameter<String>(
				"extraFilesDirectory",
				"Import files to",
				"Select a directory where all files generated by " +
				"the program are saved.",
				new PathParameterType(Path.Type.DIRECTORY, homeDirectoryPath, true)
			));
			
			configureJob = new RequestInformation(
					command,
					"Select import options.",
					"",
					parameters
				);
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return configureJob;
	}
	
	private Directory getExtraFilesDirectory(DbControl dc, String pathName)
	{ 
		Path path = new Path(pathName, Path.Type.DIRECTORY);
		Directory dir = null;
		try
		{
			// Load existing...
			dir = Directory.getByPath(dc, path);
		}
		catch(ItemNotFoundException e)
		{
			// Create a new directory if no existing were found
			dir = Directory.getNew(dc, path);
			dc.saveItem(dir);
		}
		return dir;
	}

	
}
