/*
	$Id: TestLabeledExtract.java 5340 2010-05-10 06:15:00Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
import net.sf.basedb.core.*;

import java.util.Date;
import java.util.Set;

public class TestLabeledExtract
{

	static boolean ok = true;
	public static void main(String[] args)
	{
		TestUtil.checkArgs(args);
		TestUtil.begin();
		ok = test_all();
		TestUtil.stop();
	}

	static boolean test_all()
	{
		write("++Testing labeled extracts");
		write_header();
		// Standard tests: create, load, list
		int labelId = TestLabel.test_create(true);
		int id = test_create(0, labelId, true);
		int id2 = test_create(0, labelId, false);
		
		int extractId = TestExtract.test_create(0, false);
		int id3 = test_create(extractId, labelId, false);
		int id4 = test_create_pooled(labelId, id, id2, id3);

		test_load(id);
		test_list(-1);
		
		write_events_header();
		test_add_event(id, 100.0f);
		test_list_events(id, 3);  // creation event, pooling event for id4, and extra event
		
		write_sources_header();
		test_list_sources(id3, 1); // The extract
		test_list_sources(id4, 3); // The three id, id2 and id3 labeled extracts
		test_list_pooledChildren(id2, 1);
		test_list_hybridizations(id, 0);

		// Standard test: Delete
		if (TestUtil.waitBeforeDelete()) TestUtil.waitForEnter();
 		test_delete(id4);
 		test_delete(id3);
 		test_delete(id2);
 		test_delete(id);
 		TestExtract.test_delete(extractId);
		TestLabel.test_delete(labelId);
	
		write("++Testing labeled extracts "+(ok ? "OK" : "Failed")+"\n");
		return ok;
	}

	static int test_create(int extractId, int labelId, boolean setAll)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.LABELEDEXTRACT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Label l = Label.getById(dc, labelId);
			LabeledExtract le = LabeledExtract.getNew(dc, l);
			if (setAll)
			{
				le.setName("Test labeled extract");
				le.setDescription("Added at "+new Date());
				le.setExternalId("XCV-455-LABELEDEXTRACT");
				le.setOriginalQuantity(500.0f);
			}
			if (extractId != 0)
			{
				Extract e = Extract.getById(dc, extractId);
				le.setExtract(e, 150.0f);
				le.setName(e.getName() + ".le");
			}
			dc.saveItem(le);
			dc.commit();
			id = le.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(le, false);
			write_item(0, le);
			write("--Create labeled extract OK");
		}
		catch (Throwable ex)
		{
			write("--Create labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}

	static int test_create_pooled(int labelId, int... sourceIds)
	{
		if (!TestUtil.hasPermission(Permission.CREATE, Item.LABELEDEXTRACT)) return 0;
		int id = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			Label l = Label.getById(dc, labelId);
			LabeledExtract le = LabeledExtract.getNew(dc, l);
			le.setName("Test pooled labeled extract");
			le.setDescription("Added at "+new Date());
			le.setExternalId("POOLED-455-LABELEDEXTRACT");
			le.setOriginalQuantity(500.0f);
			le.setPooled(true);
			BioMaterialEvent evt = le.getCreationEvent();
			for (int sourceId : sourceIds)
			{
				LabeledExtract src = LabeledExtract.getById(dc, sourceId);
				Float original = src.getOriginalQuantity();
				evt.addSource(src, original == null ? null : original/2);
			}
			dc.saveItem(le);
			dc.commit();
			id = le.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(le, false);
			write_item(0, le);
			write("--Create pooled labeled extract OK");
		}
		catch (Throwable ex)
		{
			write("--Create pooled labled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return id;
	}
	
	static void test_load(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, id);
			write_item(0, le);
			write("--Load labeled extract OK");
		}
		catch (Throwable ex)
		{
			write("--Load labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list(int expectedResults)
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<LabeledExtract> l = LabeledExtract.getQuery().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List labeled extracts OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List labeled extracts FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_delete(int id)
	{
		if (id == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, id);
			dc.deleteItem(le);
			Set<ItemProxy> using = le.getUsingItems();
			if (using.size() > 0) 
			{
			   throw new BaseException(le + " is used by " + using);
			}
			dc.commit();
			write("--Delete labeled extract OK");
		}
		catch (Throwable ex)
		{
			write("--Delete labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_delete_all()
	{
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			ItemResultList<LabeledExtract> l = LabeledExtract.getQuery().list(dc);
			Trashcan.delete(dc.getSessionControl(), l, true, null);
			dc.commit();
			write("--Delete all labeled extracts OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--Delete all labeled extracts FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void write_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal id\tExtract\tLabel\tQuantity\tRemain");
			write("-- \t-- \t--------- \t-----------\t-----------\t-------\t-----\t--------\t------");
		}
	}
	static void write_item(int i, LabeledExtract le)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+le.getId()+"\t"+le.getName()+"\t"+le.getDescription()+
			"\t"+le.getExternalId()+"\t"+le.getExtract()+"\t"+le.getLabel()+"\t"+
			le.getOriginalQuantity()+"\t"+le.getRemainingQuantity());
	}
	static void write_events_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tBiomaterial\tEvent type\tEvent date\tEntry date\tUsed quantity\tComment");
			write("-- \t-- \t-----------\t----------\t----------\t----------\t-------------\t-------");
		}
	}
	static void write_item(int i, BioMaterialEvent evt, LabeledExtract le)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+evt.getId()+"\t"+evt.getBioMaterial()+"\t"+evt.getEventType()+
			"\t"+evt.getEventDate()+"\t"+evt.getEntryDate()+"\t"+evt.getUsedQuantity(le)+"\t"+evt.getComment());
	}
	static void write_sources_header()
	{
		if (!TestUtil.getSilent())
		{
			write("   \tID \tName      \tDescription\tExternal id\tPooled\tQuantity\tRemain\tUsed");
			write("-- \t-- \t--------- \t-----------\t-----------\t------\t--------\t------\t----");
		}
	}
	static void write_item(int i, MeasuredBioMaterial b, BioMaterialEvent evt)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+b.getId()+"\t"+b.getName()+"\t"+b.getDescription()+
			"\t"+b.getExternalId()+"\t"+b.isPooled()+"\t"+b.getOriginalQuantity()+"\t"+b.getRemainingQuantity()+
			"\t"+evt.getUsedQuantity(b));
	}
	static void write_item(int i, Hybridization h)
		throws BaseException
	{
		if (!TestUtil.getSilent()) System.out.println(i+":\t"+h.getId()+"\t"+h.getName()+"\t"+h.getDescription());
	}
	static void write(String message)
	{
		System.out.println(message);
	}
	
	static int test_add_event(int labeledExtractId, float usedQuantity)
	{
		if (labeledExtractId == 0) return 0;
		int eventId = 0;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, labeledExtractId);
			BioMaterialEvent bme = le.newEvent();
			bme.setUsedQuantity(usedQuantity);
			bme.setComment("Added a new event");
			dc.saveItem(bme);
			dc.commit();
			eventId = bme.getId();
			dc = TestUtil.getDbControl();
			dc.reattachItem(bme, false);
			write_item(0, bme, le);
			write("--Add event for labeled extract OK");
		}
		catch (Throwable ex)
		{
			write("--Add event for labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
		return eventId;
	}
	
	static void test_list_events(int labeledExtractId, int expectedResults)
	{
		if (labeledExtractId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, labeledExtractId);
			ItemResultList<BioMaterialEvent> l = le.getEvents().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i), le);
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List events for labeled extract OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List events for labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}

	static void test_list_sources(int labeledExtractId, int expectedResults)
	{
		if (labeledExtractId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, labeledExtractId);
			BioMaterialEvent evt = le.getCreationEvent();
			ItemResultList<? extends MeasuredBioMaterial> l = evt.getSources().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i), evt);
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List sources for labeled extract OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List sources for labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_hybridizations(int labeledExtractId, int expectedResults)
	{
		if (labeledExtractId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, labeledExtractId);
			ItemResultList<Hybridization> l = le.getHybridizations().list(dc);
			for (int i = 0; i<l.size(); i++)
			{
				write_item(i, l.get(i));
			}
			if (expectedResults >= 0 && expectedResults != l.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+l.size());
			}
			write("--List hybridizations for labeled extract OK ("+l.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List hybridizations for labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
	
	static void test_list_pooledChildren(int labeledExtractId, int expectedResults)
	{
		if (labeledExtractId == 0) return;
		DbControl dc = null;
		try
		{
			dc = TestUtil.getDbControl();
			LabeledExtract le = LabeledExtract.getById(dc, labeledExtractId);
			ItemResultList<BioMaterialEvent> ev = le.getPoolingEvents().list(dc);
			
			for (int i=0; i<ev.size(); i++)
			{
				write_item(i, (LabeledExtract)ev.get(i).getBioMaterial());
			}
			if (expectedResults >= 0 && expectedResults != ev.size())
			{
				throw new BaseException("Expected "+expectedResults+" results, not "+ev.size());
			}
			write("--List pools from labeledExtract OK ("+ev.size()+")");
		}
		catch (Throwable ex)
		{
			write("--List pools for labeled extract FAILED");
			ex.printStackTrace();
			ok = false;
		}
		finally
		{
			if (dc != null) dc.close();
		}
	}
}
