/* $Id: linkitems.js 4889 2009-04-06 12:52:39Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	JavaScript functions for managing many-to-many
	links between items. To use this script library a page
	should use a single <select> list box for displaying the 
	currently linked items. This library supports having more than
	one type of items in the same list if it is divided into
	sections.
	
	The value of the options initially in the list must be constructed
	as follows:
	
	* For a section header the value must be a single letter type code.
	  For example, 'U' for users and 'G' for grups.
	* For each item in a section the value must be the same type code
	  plus the item's id. For example 'U123' or 'G55'.
	  
	 When page has loaded the Link.initList must be called so that
	 an internal information structure can be built from the items
	 in the list.

	@author Nicklas
	@version 2.0
*/

var Link = new LinkClass();
function LinkClass()
{
	/**
		Add a new item to the end of a list
		@param list The list object
		@param item The Item object
	*/
	this.addNewItem = function(list, item)
	{
		var option = new Option(item.name, item.type+item.id);
		Forms.addListOption(list, list.length, option);
		option.item = item;
	}
	
	/**
		Add a new section to the end of a list
		@param list The list object
		@param section The Section object
	*/
	this.addNewSection = function(list, section)
	{
		var option = new Option('-- '+section.name+' --', section.type);
		Forms.addListOption(list, list.length, option);
		option.className = 'selectoptionheader';
		option.section = section;
		list.hasSections = true;
	}

	this.getItem = function(type, id)
	{
		var item = allItems['T'+type+id];
		return item;
	}
	
	/**
		Add an item to the list. If the item already is in the
		list, it is selected, otherwise a new option is inserted and
		an action to create the link is registered.
		@param list The list object
		@param item An item object
	*/
	this.addItem = function(list, item)
	{
		var sectionFound = !list.hasSections;
		var insertIndex = list.length;
		for (var i = 0; i < list.length; i++)
		{
			var listItem = list[i].item;
			var section = list[i].section;
			if (!sectionFound && section)
			{
				// Have we found the correct section?
				sectionFound = section.type == item.type;
			}
			if (sectionFound)
			{
				if (section && section.type != item.type)
				{
					// End of correct section, insert new item here
					insertIndex = i;
					i = list.length;
				}
				else if (listItem && listItem.type == item.type && listItem.id == item.id)
				{
					// The item is already in the list, select it and return
					list[i].selected = true;
					return;
				}
			}
		}
		if (!sectionFound)
		{
			alert('Error: Could not find a section for type '+item.type);
		}
		else
		{
			var option = new Option(item.name, item.type+item.id, false, true);
			Forms.addListOption(list, insertIndex, option);
			option.item = item;
			item.action++;
		}
	}
	
	/**
		Remove all selected option from the list. Actions to remove
		the links are also registered.
		@param list The list object
		@param addTo An optional list item to add the removed items to
	*/
	this.removeSelected = function(list, addTo)
	{
		for (var i = 0; i < list.length; i++)
		{
			var listItem = list[i].item;
			if (list[i].selected && listItem && listItem.id)
			{
				listItem.action--;
				listItem.value = 0;
				list[i--] = null;
				if (addTo)
				{
					addTo[addTo.length] = new Option(listItem.name, listItem.id, false, true);
				}
			}
		}
	}
	
	/**
		Get the IDs for all item with the specified action and type
		@param action The action, -1 = remove, 1 = add, 0 = do nothing
		@param type The type code
	*/
	this.getActionIds = function(action, type)
	{
		var ids = new Array();
		for (var i = 0; i < allItems.length; i++) // >
		{
			var item = allItems[i];
			if (item.type == type && item.action == action)
			{
				ids[ids.length] = item.id;
			}
		}
		return ids;
	}
	
	/**
		Get the IDs for all items of the specified type in the list
		@param list The list to search
		@param type The type code
	*/
	this.getListIds = function(list, type)
	{
		var ids = new Array();
		for (var i = 0; i < list.length; i++)
		{
			var listItem = list[i].item;
			if (listItem && listItem.type == type && listItem.id)
			{
				ids[ids.length] = listItem.id;
			}
		}
		return ids;
	}
	
	this.exportModified = function(frm, type, ignoreRemoved)
	{
		var ids = new Array();
		for (var i = 0; i < allItems.length; i++) // >
		{
			var item = allItems[i];
			if (item.type == type && (item.value != item.originalValue || item.action > 0 || (!ignoreRemoved && item.action < 0)))
			{
				ids[ids.length] = item.id;
				Forms.createHidden(frm, item.type+item.id, item.value);
			}
		}
		return ids;
	}
}

var allItems = new Array();
/**
	Create a new Item object.
	@param type The type code
	@param id The item's ID
	@param name The display name of the item
*/
function Item(type, id, name, value, originalValue)
{
	this.type = type;
	this.id = id;
	this.name = name;
	this.value = value;
	this.originalValue = originalValue == undefined ? value : originalValue;
	this.action = 0;
	allItems[allItems.length] = this;
	allItems['T'+this.type+this.id] = this;
}
function Section(type, name)
{
	this.type = type;
	this.name = name;
}

